/*
   Copyright (C) 1998 T. Scott Dattalo

This file is part of gpsim.

gpsim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

gpsim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with gpasm; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */


#ifndef  __BREAKPOINTS_H__
#define  __BREAKPOINTS_H__

#include "instruction.h"
#include "register.h"
#include "glibconfig.h"
#include <iostream>
#include <unistd.h>

/*! \mainpage MicroSim - PIC16F84A Simulator
   
   \section Introduction 
   MicroSim is a software simulator of the popular PIC16F84A microcontroller
   from MicroChip. It is built on the core of Scott Dattalo's gpsim,
   which is an open source simulator available from SourceForge. MicroSim
   offers an intuitive graphical user interface that logically groups program
   views into appropriate tabs for efficient PIC program debugging and development.
   An assembled PIC program can be loaded into the simulator and run or stepped 
   through to observe it's behaviour, before committing the program to a phyiscal
   microcontroller chip. MicroSim also offers interaction with the parallel port.
   The IO ports of the PIC16F84A are simulated via the parallel port pins so that
   application circuitry can be attached to and driven by the parallel port.
   MicroSim performs all this useful functionality at speeds matching the
   actual PIC operation speed (pseudo realtime). 
*/

//class Invalid_File_Register;

//! \def #define for maximum breakpoints
/*!
 *  \brief This sets the maximum breakpoints to be 0x400
 */
#define MAX_BREAKPOINTS 0x400

//! \def #define for the breakpoint mask
/*!
 *  \brief Sets the breakpoint mask for processing breakpoints
 */
#define BREAKPOINT_MASK (MAX_BREAKPOINTS-1)

//! \class 
/*!
 *  \brief This class models a breakpoint instruction
 *
 *  Inherits from base class Instruction, it replaces a normal
 *  instruction in program memory with an object that can
 *  trigger a global halt (breakpoint).
 */
class Breakpoint_Instruction : public Instruction
{
public:

  //! Pointer to the processor object
  Processor *cpu;
  //! The address of the instruction        
  unsigned int address; 
  //! The number of the breakpoint 
  unsigned int bpn;   
  //! Pointer to the replaced instruction   
  Instruction *replaced; 

  //! Gets the opcode of the breakpoint instruction
  /*!
      \return The opcode of instruction
  */
  virtual unsigned int get_opcode(void);
  
  //! Gets the source line associated with instruction
  /*!
      \return The source line number
  */
  virtual int get_src_line(void);
  
  //! Gets the HLL source line associated with instruction
  /*!
      \return HLL source line number
  */
  virtual int get_hll_src_line(void);
  
  //! Gets the LST file line
  /*!
      \return The LST file line number
  */
  virtual int get_lst_line(void);
  
  //! Gets the file ID of source
  /*!
      \return The file ID
  */
  virtual int get_file_id(void);
  
  //! Gets the file ID of HLL source
  /*!
      \return The file ID
  */
  virtual int get_hll_file_id(void);
  
  //! Breakpoint_Instruction constructor
  /*!
      Creates a new Breakpoint_Instruction
      \param new_cpu Pointer to the processor object
      \param new_address The address of the instruction
      \param bp The number of the breakpoint
  */
  Breakpoint_Instruction(Processor *new_cpu, unsigned int new_address, unsigned int bp);
  
  //! Returns the type of instruction
  /*!
      \return Instruction type (BREAKPOINT_INSTRUCTION)
  */
  virtual INSTRUCTION_TYPES isa(void) {return BREAKPOINT_INSTRUCTION;};
  
  //! Executes the instruction
  /*!
      This executes the instruction and sets a global halt because
      a breakpoint has been reached.
  */
  virtual void execute(void);
  
  //! Gets name of the instruction
  /*!
      Returns the name of the instruction (eg. MOVW)
  */
  virtual char *name(char* return_str);

};

//! \class 
/*!
    \brief Notifies objects that a break has occured
 
    This class is intended to be one of the classes in
    in a multiple inheritance class declaration. It's purpose is to provide
    a mechanism by which a break point can notify an object that the break
    has occurred.
    Note that BreakCallBack is typically used in conjunction with Cycle
    Counter break points (see the Cycle_Counter class). 
*/
class BreakCallBack
{
public:
  //! BreakCallBack constructor
  /*!
      Default constructor for BreakCallBack
  */
  BreakCallBack() {};
  //! A unique number assigned when the break point is armed.
  int CallBackID;  

  //! Makes a callback to another object
  /*!
      Notifies an associated object that a break has occured
  */
  virtual void callback(void);

  //! Makes a callback but displays info 
  /*!
      Invoked by Cycle counter to display info about break call back.
  */
  virtual void callback_print(void);

  //! Clears the BreakCallBack
  /*!
      clear_break is invoked when a BreakCallBack object's
      associated break point is cleared. 
  */
  virtual void clear_break(void);
};

class Notify_Instruction : public Breakpoint_Instruction
{
    BreakCallBack *callback;
public:
    Notify_Instruction(Processor *cpu, unsigned int address, unsigned int bp, BreakCallBack *cb);
    virtual INSTRUCTION_TYPES isa(void) {return NOTIFY_INSTRUCTION;};
    virtual void execute(void);
};

class Profile_Start_Instruction : public Notify_Instruction
{
public:
    Profile_Start_Instruction(Processor *cpu, unsigned int address, unsigned int bp, BreakCallBack *cb);
    virtual INSTRUCTION_TYPES isa(void) {return PROFILE_START_INSTRUCTION;};
};

class Profile_Stop_Instruction : public Notify_Instruction
{
public:
    Profile_Stop_Instruction(Processor *cpu, unsigned int address, unsigned int bp, BreakCallBack *cb);
    virtual INSTRUCTION_TYPES isa(void) {return PROFILE_STOP_INSTRUCTION;};
};


//! \class 
/*!
    \brief This class handles breakpoints globally
    
    A global instance of this class is used to handle all breakpoints
    functionality. It is able to set and clear instruction and register
    breakpoints. It can perform other breakpoints operations which are not
    yet used by MicroSim, but were useful in gpsim.
*/
class Breakpoints
{
public:

//! \enum 
/*!    
    \brief This specifies the various breakpoint types that
    are handled.
*/
enum BREAKPOINT_TYPES
{
  //! No break
  BREAK_CLEAR          = 0,
  //! Break on execute
  BREAK_ON_EXECUTION   = 1<<24,
  //! Break on reg read
  BREAK_ON_REG_READ    = 2<<24,
  //! Break on reg write
  BREAK_ON_REG_WRITE   = 3<<24,
  //! Break on reg read value
  BREAK_ON_REG_READ_VALUE  = 4<<24,
  //! Break on reg write value
  BREAK_ON_REG_WRITE_VALUE = 5<<24,
  //! Break on invalid register
  BREAK_ON_INVALID_FR  = 6<<24,
  //! Break on cycle
  BREAK_ON_CYCLE       = 7<<24,
  //! Break on WDT timeout
  BREAK_ON_WDT_TIMEOUT = 8<<24,
  //! Break on stack overflow
  BREAK_ON_STK_OVERFLOW  = 9<<24,
  //! Break on stack underflow
  BREAK_ON_STK_UNDERFLOW = 10<<24,
  //! Notify on execute
  NOTIFY_ON_EXECUTION   = 11<<24,
  //! Profile start notify on execute
  PROFILE_START_NOTIFY_ON_EXECUTION = 12<<24,
  //! Profile stop notify on execute
  PROFILE_STOP_NOTIFY_ON_EXECUTION = 13<<24,
  //! Notify on reg read
  NOTIFY_ON_REG_READ        = 14<<24,
  //! Notify on reg write
  NOTIFY_ON_REG_WRITE       = 15<<24,
  //! Notify on reg read value
  NOTIFY_ON_REG_READ_VALUE  = 16<<24,
  //! Notify on reg write value
  NOTIFY_ON_REG_WRITE_VALUE = 17<<24,
  //! Break mask
  BREAK_MASK           = 0xff<<24
};

//! \define Indicates a global clear
#define  GLOBAL_CLEAR         0
//! \define Indicates a global execution stop
#define  GLOBAL_STOP_RUNNING  1
//! \define Indicates a global interrupt
#define  GLOBAL_INTERRUPT     2
//! \define Indicates a global sleep
#define  GLOBAL_SLEEP         4
//! \define Indicates a global PM write
#define  GLOBAL_PM_WRITE      8

//! \struct
/*!    
    \brief This is associated with a breakpoint to store
    it's status.
*/
struct BreakStatus
{
  //! Type of breakpoint
  BREAKPOINT_TYPES type; 
  //! The processor
  Processor *cpu;     
  //! Arg1   
  unsigned int arg1; 
  //! Arg2  
  unsigned int arg2;  
  //! BreakCallBack object   
  BreakCallBack *f;      
} break_status[MAX_BREAKPOINTS];

  //! Indicates whether there is a global break
  unsigned int  global_break;  
  //! The breakpoint number
  unsigned int breakpoint_number; 
  // The last breakpoint
  unsigned int last_breakpoint;


  //! Default Constructor
  /*
      Creates a new Breakpoints object
  */
  Breakpoints(void);
  
  //! Sets a breakpoint
  /*
      Sets a breakpoint at a given address
      \param break_type Type of breakpoint
      \param cpu The processor object
      \param arg1 Arg1
      \param arg2 Arg2
      \param f Pointer to BreakCallBack
  */
  unsigned int set_breakpoint(BREAKPOINT_TYPES,Processor *,unsigned int, unsigned int,BreakCallBack *f = NULL);
  
  //! Sets an execution breakpoint
  /*
      \param cpu The processor object
      \param address The address of breakpoint
  */
  unsigned int set_execution_break(Processor *cpu, unsigned int address);
  
  //! Sets a notify breakpoint
  /*
      \param cpu The processor object
      \param address The address of notify breakpoint
      \param cb Pointer to BreakCallBack
  */
  unsigned int set_notify_break(Processor *cpu, unsigned int address, BreakCallBack *cb);
  
  unsigned int set_profile_start_break(Processor *cpu, unsigned int address, BreakCallBack *f1 = NULL);
  unsigned int set_profile_stop_break(Processor *cpu, unsigned int address, BreakCallBack *f1 = NULL);
  
  //! Sets a register read break
  /*
      \param cpu The processor object
      \param register_number The address of register
  */
  unsigned int set_read_break(Processor *cpu, unsigned int register_number);
  
  //! Sets a register write break
  /*
      \param cpu The processor object
      \param register_number The address of register
  */
  unsigned int set_write_break(Processor *cpu, unsigned int register_number);
  unsigned int set_read_value_break(Processor *cpu, unsigned int register_number, unsigned int value, unsigned int mask=0xff);
  unsigned int set_write_value_break(Processor *cpu, unsigned int register_number, unsigned int value, unsigned int mask=0xff);
  unsigned int set_cycle_break(Processor *cpu, guint64 cycle,BreakCallBack *f = NULL);
  unsigned int set_wdt_break(Processor *cpu);
  unsigned int set_stk_overflow_break(Processor *cpu);
  unsigned int set_stk_underflow_break(Processor *cpu);
  unsigned int check_write_break(Register *fr);
  unsigned int check_read_break(Register *fr);
  unsigned int check_break(Register *fr);
  unsigned int check_invalid_fr_break(Invalid_File_Register *fr);
  unsigned int check_cycle_break(unsigned int abp);
  unsigned int set_notify_read(Processor *cpu, unsigned int register_number);
  unsigned int set_notify_write(Processor *cpu, unsigned int register_number);
  unsigned int set_notify_read_value(Processor *cpu, unsigned int register_number, unsigned int value, unsigned int mask=0xff);
  unsigned int set_notify_write_value(Processor *cpu, unsigned int register_number, unsigned int value, unsigned int mask=0xff);

  //! Clears a global break
  inline void clear_global(void) {global_break = GLOBAL_CLEAR;};
  //! Causes a halt
  void halt(void);
  
  //! Indicates whether a halt has occured
  /*
      \return True is halt occured, false otherwise
  */
  inline bool have_halt(void) { return( (global_break & GLOBAL_STOP_RUNNING) != 0 );};
  
  //! Clears a halt
  inline void clear_halt(void) {global_break &= ~GLOBAL_STOP_RUNNING;};
  
  //! Indicates whether an interrupt has occured
  /*
      \return True if interrupt occured, false otherwise
  */
  inline bool have_interrupt(void) { return( (global_break & GLOBAL_INTERRUPT) != 0 );};
  
  //! Clears interrupt
  inline void clear_interrupt(void) {global_break &= ~GLOBAL_INTERRUPT;};
  
  //! Sets a flag that indicates an interrupt has occured
  inline void set_interrupt(void) {global_break |= GLOBAL_INTERRUPT;};
  
  //! Indicates whether system is in SLEEP
  /*
      \return True if in SLEEP, false otherwise
  */
  inline bool have_sleep(void) { return( (global_break & GLOBAL_SLEEP) != 0 );};
  
  //! Clears sleep
  inline void clear_sleep(void) {global_break &= ~GLOBAL_SLEEP;};
  
  //! Sets flag that indcates SLEEP
  inline void set_sleep(void) {global_break |= GLOBAL_SLEEP;};
  inline bool have_pm_write(void) { return( (global_break & GLOBAL_PM_WRITE) != 0 );};
  inline void clear_pm_write(void) {global_break &= ~GLOBAL_PM_WRITE;};
  inline void set_pm_write(void) {global_break |= GLOBAL_PM_WRITE;};


  //bool dump1(unsigned int bp_num);
  //void dump(void);
  //void dump_traced(unsigned int b);
  
  //! Clears the breakpoint with given number
  /*
      \param b Breakpoint number
  */
  void clear(unsigned int b);
  
  //! Clears all breakpoints
  /*
      \param c Processor object
  */
  void clear_all(Processor *c);
  void clear_all_set_by_user(Processor *c);
  
  //! Clears all breakpoints set on a register
  /*
      \param c Processor object
      \param address Address of register
  */
  void clear_all_register(Processor *c,unsigned int address=-1);
  void initialize_breakpoints(unsigned int memory_size);
  Instruction *find_previous(Processor *cpu, unsigned int address, Instruction *_this);

};

extern Breakpoints bp;

//! \class
/*!
    \brief Base class for register breakpoints

    This class serves as the base class for register break point and logging
    classes. Register breakpoints are handled by replacing a register object
    with one of the breakpoint objects. The simulated pic code has no idea that
    breakpoints exist on a register. However, when the member functions of the
    a register are accessed, the breakpoint member functions of the classes
    described below are the ones actually invoked. Consequently, control of
    the simulation can be manipulated.
*/

class Notify_Register : public Register
{
public:
  //! The number associated with this break point.
  unsigned int break_point; 
  //! A pointer to the register that this break replaces
  Register *replaced; 
  //! If multiple breaks are set on one register, then this will point to the next one.       
  Notify_Register *next;    

  //! Default Constructor
  Notify_Register(void){ replaced = NULL; next = NULL;};
  //! Notify_Register constructor
  /*!
      \param cpu Processor object
      \param _repl The instruction replaced
      \param bp Breakpoint number
  */
  Notify_Register(Processor *cpu, int _repl, int bp);

  //! Returns the type of register
  /*!
     \return The type of register (BP_REGISTER)
  */
  virtual REGISTER_TYPES isa(void) {return BP_REGISTER;};
  
  //! Returns the name of register
  /*!
      \return Name of register
  */
  virtual char *name(void)
    {
      if(replaced)
    return replaced->name();
  return NULL;
    };
    
  /* direct all accesses to the member functions of the
   * register that is being replaced. Note that we assume
   * "replaced" is properly initialized which it will be
   * if this object is accessed. (Why? well, we only access
   * register notify/breaks via the PIC's file register 
   * memory and never directly access them. But the only
   * way this instantiation can be accessed is if it successfully
   * replaced a file register object */

  //! Puts a value into register
  /*!
      \param new_value The value
  */
  virtual void put_value(unsigned int new_value)
    {
      replaced->put_value(new_value);
    }
  //! Puts a value into register
  /*!
      \param new_value The value
  */
  virtual void put(unsigned int new_value)
    {
      replaced->put_value(new_value);
    }
  //! Gets value of register
  /*!
      \return The value
  */
  virtual unsigned int get_value(void)
    {
      return(replaced->get_value());
    }
  //! Gets value of register
  /*!
      \return The value
  */
  virtual unsigned int get(void)
    {
      return(replaced->get_value());
    }

  //! Gets the replaced register
  /*!
      \return The register that has been replaced
  */
  virtual Register *getReg(void)
    {
      if(replaced)
    return replaced;
      else
    return this;
    }

  //! Sets a bit in the register
  /*!
      \param bit_number Bit to set
      \param new_value True for 1 or false for 0
  */
  virtual void setBit(unsigned int bit_number, bool new_value)
    {
      replaced->setBit(bit_number, new_value);
    }

  //! Gets a bit in the register
  /*!
      \param bit_number Bit to get
      \return The value of bit 
  */
  virtual int getBit(unsigned int bit_number)
    {
      return(replaced->getBit(bit_number));
    }

  /*virtual int get_bit_voltage(unsigned int bit_number)
    {
      return(replaced->get_bit_voltage(bit_number));
    }*/

  virtual bool hasBreak(void)
    { 
      return true;
    }

  //! Replaces the register with a Notify_Register
  /*!
      \param _cpu Processor object
      \param reg The register to replace
  */
  void replace(Processor *_cpu, unsigned int reg);
  
  //! Clears the register of breakpoint
  /*!
      \param bp_num Breakpoint to clear
  */
  unsigned int clear(unsigned int bp_num);

};

class Notify_Register_Value : public Notify_Register
{
public:

  unsigned int break_value, break_mask, last_value;

  Notify_Register_Value(void)
    { 
      replaced = NULL;
      break_value = 0;
      break_mask = 0;
      last_value = 0;
    }

  Notify_Register_Value(Processor *_cpu, int _repl, int bp, int bv, int bm ):
    Notify_Register(_cpu,_repl,bp ) 
    { 
      break_value = bv;
      break_mask = bm;
      if(replaced)
    last_value = replaced->get_value();

    };

};


//! \class
/*!
    \brief Extends Notify_Register to model a register read break
*/
class Break_register_read : public Notify_Register
{
public:
  //! Default constructor
  Break_register_read(void){ };
  //! Break_register_read constructor
  /*!
      Creates a Break_register read object.
      \param _cpu Processor object
      \param _repl Register to replace
      \param bp Breakpoint number
  */
  Break_register_read(Processor *_cpu, int _repl, int bp ):
  Notify_Register(_cpu,_repl,bp ) { };

  //! Gets the value of a register and sets a global break
  /*!
      \return Value of register
  */
  virtual unsigned int get_value(void);
  
  //! Gets a bit from the register and sets global break
  /*!
      \return Value of bit in register
  */
  virtual int getBit(unsigned int bit_number);
  //virtual int get_bit_voltage(unsigned int bit_number);

};

//! \class 
/*!
    \brief Extends Notify_Register to model a register write break
*/
class Break_register_write : public Notify_Register
{
public:
  //! Default constructor
  Break_register_write(void){ };
  //! Break_register_write constructor
  /*!
      Creates a Break_register read object.
      \param _cpu Processor object
      \param _repl Register to replace
      \param bp Breakpoint number
  */
  Break_register_write(Processor *_cpu, int _repl, int bp ):
  Notify_Register(_cpu,_repl,bp ) { };
  
  //! Puts a value into register and sets global break
  /*!
      \param new_value Value to put
  */
  virtual void put_value(unsigned int new_value);
  
  //! Sets a bit in the register and sets global break
  /*!
      \param bit_number Bit to set
      \param new_value True for 1, false for 0
  */
  virtual void setBit(unsigned int bit_number, bool new_value);

};

class Break_register_read_value : public Notify_Register_Value
{
public:

  Break_register_read_value(void){ };
  Break_register_read_value(Processor *_cpu, int _repl, int bp, int bv, int bm ) :
    Notify_Register_Value(_cpu,  _repl, bp, bv, bm ) { };

  virtual unsigned int get(void);
  virtual int getBit(unsigned int bit_number);
  //virtual int get_bit_voltage(unsigned int bit_number);

};

class Break_register_write_value : public Notify_Register_Value
{
public:

  Break_register_write_value(void){ };
  Break_register_write_value(Processor *_cpu, int _repl, int bp, int bv, int bm ) :
    Notify_Register_Value(_cpu,  _repl, bp, bv, bm ) { };

  virtual void put(unsigned int new_value);
  virtual void setBit(unsigned int bit_number, bool new_value);
};

class Log_Register_Write : public Notify_Register
{
 public:

  Log_Register_Write(void){ };
  Log_Register_Write(Processor *_cpu, int _repl, int bp ):
    Notify_Register(_cpu,_repl,bp ) { };
  virtual void put(unsigned int new_value);
  virtual void setBit(unsigned int bit_number, bool new_value);

};

class Log_Register_Read : public Notify_Register
{
public:


  Log_Register_Read(void){ };
  Log_Register_Read(Processor *_cpu, int _repl, int bp ):
    Notify_Register(_cpu,_repl,bp ) { };
  virtual unsigned int get(void);
  virtual int getBit(unsigned int bit_number);
  //virtual int get_bit_voltage(unsigned int bit_number);
};

class Log_Register_Read_value : public Notify_Register_Value
{
public:

  Log_Register_Read_value(void){ };
  Log_Register_Read_value(Processor *_cpu, int _repl, int bp, int bv, int bm ) :
    Notify_Register_Value(_cpu,  _repl, bp, bv, bm ) { };
  virtual unsigned int get(void);
  virtual int getBit(unsigned int bit_number);
  //virtual int get_bit_voltage(unsigned int bit_number);
};

class Log_Register_Write_value : public Notify_Register_Value
{
public:

  Log_Register_Write_value(void){ };
  Log_Register_Write_value(Processor *_cpu, int _repl, int bp, int bv, int bm ) :
    Notify_Register_Value(_cpu,  _repl, bp, bv, bm ) { };

  virtual void put(unsigned int new_value);

};

/*
#ifdef HAVE_GUI
class GuiCallBack: public BreakCallBack
{
public:
  virtual void callback(void);

  gpointer gui_callback_data;  // Data to be passed back to the gui

  // A pointer to the gui call back function
  void  (*gui_callback) (gpointer gui_callback_data);
  void set_break(int, void (*)(gpointer),gpointer );

  GuiCallBack(void);
};
#endif // HAVE_GUI
*/
//--------------------------------------------------------------------
// InterfaceObject 
//
// This class is used only within interface.cc's scope. It's purpose
// is to provide a way to call back some function not gpsim's code
// space.
/*
class InterfaceObject : public BreakCallBack
{
public:
  Processor *pic;
  gpointer callback_data;
  void (*callback_function)(gpointer);
  

  InterfaceObject(void) {pic = NULL;};
  virtual void callback(void);

};*/

//------------------------------------------------------------------------
// Primarily for interface.cc
class CyclicBreakPoint : public BreakCallBack
{
public:
  guint64 delta_cycles;

  void set_break(void);
  virtual void callback(void);
  virtual ~CyclicBreakPoint(void);
  void set_delta(guint64 delta);
};


#endif   //  __BREAKPOINTS_H__
