#include "glibconfig.h"
//#include "breakpoints.h"
#include <cstdlib>
#ifndef __CYCLES_H__
#define __CYCLES_H__

#define MAX_BREAKPOINTS 0x400
class BreakCallBack;

//---------------------------------------------------------
//! \class
/*!
    \brief Cycle Counter breakpoint list to break on cycles
    This class is used to implement a very simple doubly-linked-list.
    See Cycle_Counter::set_break for more details on how it is used.
*/
class Cycle_Counter_breakpoint_list
{
public:
  //! Pointer to next
  Cycle_Counter_breakpoint_list *next; 
  //! Pointer to previous 
  Cycle_Counter_breakpoint_list *prev; 

  //! Break value
  guint64 break_value;
  //! System bp, This is used if the user set a bp.
  /*!
      It tells which system bp needs to be cleared
      when the cycle break is hit. If a peripheral set
      a cycle break, then there is no corresponding 
      system break.
  */
  unsigned int breakpoint_number;  
				 
   //! Pointer to BreakCallBack  
  BreakCallBack *f;
};

//---------------------------------------------------------
//! \class
/*!
    \brief Cycle Counter class for timing
    The cycle counter class is used to coordinate the timing 
    between the different peripherals within a processor and
    in some cases, the timing between several simulated processors
    and modules.
*/
class Cycle_Counter
{
public:

//! \define Size of Break Array
#define BREAK_ARRAY_SIZE  32
//! \define Break Array Mask
#define BREAK_ARRAY_MASK  (BREAK_ARRAY_SIZE -1)

  //! Current value of the cycle counter.
  guint64 value;       
  //! If there's a pending cycle break point, then it'll be this  
  guint64 break_on_this;  

  //! Set true when a break point is reassigned (or deleted)
  bool reassigned;        

  //! Head of the active breakpoint linked list
  Cycle_Counter_breakpoint_list active; 
  //! Head of the inactive one.   
  Cycle_Counter_breakpoint_list inactive;  

  //! a flag that's true when the time per counter tick is constant
  bool bSynchronous; 

  //! The number of cycles that correspond to one second i.e. this is the frequency.
  double cycles_per_second; 

  //! Default Constructor
  /*!
      Default constructor to create a Cycle_Counter object
  */
  Cycle_Counter(void);
  void preset(guint64 new_value);     //! not used currently.

 private:
  //! When the member function "increment()" encounters a break point, breakpoint() is called.
  void breakpoint(void);
    

 public:
  //! Increments the cycle counter
  /*!
      increment - This inline member function is called once or 
      twice for every simulated instruction. Its purpose is to
      increment the cycle counter using roll over arithmetic.
      If there's a breakpoint set on the new value of the cycle
      counter then the simulation is either stopped or a callback
      function is invoked. In either case, the break point is
      cleared.
  */
  void increment(void);
  
  //!
  /*! Advances the cycle counter
      Advance the Cycle Counter by more than one instruction quantum.
      This is almost identical to the increment() function except that
      we allow the counter to be advanced by an arbitrary amount.
      They're separated only for efficiency reasons. This one runs slower.
      \param step The number of steps to advance by
  */ 
  void advance(guint64 step);
    
  //! Set a cycle counter break point.
  /*!
      The break points are stored in a singly-linked-list sorted by the
      order in which they will occur. When this routine is called, the
      value of 'future_cycle' is compared against the values in the
      'active' list.
      \param future_cycle The cycle to set breakpoint 
      \param f BreakCallBack object
      \param abp Breakpoint number
      \return Return true if successful, false otherwise
  */
  bool set_break(guint64 future_cycle,
		 BreakCallBack *f=NULL, unsigned int abp = MAX_BREAKPOINTS);
  //! Set a cycle counter break point relative to the current cpu cycle value. 
  /*!
      \return Return true if successful, false otherwise
  */
  bool set_break_delta(guint64 future_cycle,
		 BreakCallBack *f=NULL, unsigned int abp = MAX_BREAKPOINTS);
  //! Change the cycle of an existing break point.
  /*!
      This is only called by the internal peripherals and not (directly) by 
      the user. It's purpose is to accommodate the dynamic and unpredictable
      needs of the internal cpu timing. For example, if tmr0 is set to roll 
      over on a certain cycle and the program changes the pre-scale value, 
      then the break point has to be moved to the new cycle.
      \param old_cycle The cycle break was previously set on
      \param future_cycle The cycle to set break on
      \param f BreakCallBack object
  */
  bool reassign_break(guint64 old_cycle,guint64 future_cycle, BreakCallBack *f=NULL);
  //! Clears the current break
  void clear_current_break(void);
  //! Dumps all cyclic breakpoints to screen
  void dump_breakpoints(void);

  //! Clears break at cycle
  /*!
      \param at_cycle Cycle to clear break at
  */
  void clear_break(guint64 at_cycle);
  //! Clears break from BreakCallBack
  /*!
      \param f BreakCallBack object to clear break
  */
  void clear_break(BreakCallBack *f);
};

extern Cycle_Counter cycles;


/*
// The stopwatch object is used to keep track of the amount of
// time between events.
class StopWatch 
{
 public:
  bool enabled;
  bool count_dir;

  guint64 rollover;
  guint64 offset;
  guint64 value;

  guint64 get(void);
  double get_time(void);

  void start(void);
  void stop(void);
  void set_rollover(guint64 new_rollover);
  void set_offset(guint64 new_rollover);

  StopWatch(void);

};

extern StopWatch stop_watch;*/


#endif
