#ifndef EEPROM_H
#define EEPROM_H

#include <assert.h>

#include "register.h"
#include "breakpoints.h"
#include "processor.h"

//class Processor;
class EEPROM;
//class PIR_SET;
class INTCON;

//---------------------------------------------------------
// EECON1 - EE control register 1
//

//! \class
/*!
    \brief EE control register 1
    
    This class models the EEPROM Control Register 1
*/
class EECON1 : public SFR_Register
{
public:
//! \enum 
/*!
    \brief The various control modes of EECON1
*/
enum
{
 //! Read
 RD    = (1<<0),
 //! Write
 WR    = (1<<1),
 //! Write Enable
 WREN  = (1<<2),
 //! Write Error
 WRERR = (1<<3),
 //! EE Interrupt Flag
 EEIF  = (1<<4),
 //! EEPGD
 EEPGD = (1<<7)
};


  //! Default constructor
  /*!
      Creates an EECON1 object
  */
  EECON1(void);

  //! Puts a new value into register
  /*!
      \param new_value New value 
  */
  void put(unsigned int new_value);
  
  //! Gets the value of register
  /*!
      \return Value of register
  */
  unsigned int get(void);

  //! Sets the EEPROM reference of this EECON1
  /*!
      \param ee Pointer to EEPROM
  */
  inline void set_eeprom(EEPROM *ee) { eeprom = ee; }
  
  //! Sets the valid bits of EECON1
  /*
      \param vb Valid bits
  */
  inline void set_valid_bits(unsigned int vb) { valid_bits = valid_bits; }
  
  //! Gets the valid bits of EECON1
  /*!
      \return The valid bits
  */
  inline unsigned int get_valid_bits(void) { return (valid_bits); }
  
  //! Sets the bits of this register
  /*!
      \param b Bits to set
  */
  inline void set_bits(unsigned int b) { valid_bits |= b; }
  
  //! Clears the bits of this register
  /*!
      \param b bits to clear
  */
  inline void clear_bits(unsigned int b) { valid_bits &= ~b; }


  //private:
  unsigned int valid_bits;   //! Valid bits of the EECON1 register
  EEPROM *eeprom;            //! Pointer to associated EEPROM object
};

const unsigned int EECON1_VALID_BITS = (EECON1::RD | EECON1::WR |
  EECON1::WREN | EECON1::EEIF);

//
// EECON2 - EE control register 2
//

//! \class
/*!
    \brief This class models the EEPROM control register 2
*/
class EECON2 : public SFR_Register
{
public:

//! \enum 
/*!
    \brief Various states of the EEPROM
*/
enum EE_STATES
{
  //! Not ready state
  EENOT_READY,   
  //! Have 0x55 state       
  EEHAVE_0x55,   
  //! Ready for write state       
  EEREADY_FOR_WRITE,  
  //! Unarmed state 
  EEUNARMED,   
  //! Read state        
  EEREAD               
};

  //! Default constructor
  /*!
      Constructor to create an EECON2 object
  */
  EECON2(void);

  //! Puts a new value into register
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  
  //! Gets the value of register
  /*!
      \return Value
  */
  unsigned int get(void);
  
  //! Causes the EEPROM to reset
  /*!
      This causes a reset of the EEPROM
  */
  void ee_reset(void) { eestate = EENOT_READY;};

  //! Sets the EEPROM reference of EECON2
  /*!
      \param ee Pointer to EEPROM
  */
  inline virtual void set_eeprom(EEPROM *ee) { eeprom = ee; }
  
  //! Gets the state of the EEPROM
  /*!
      This returns the state of the EEPROM
  */ 
  inline enum EE_STATES get_eestate(void) { return (eestate); }
  
  //! Sets the EEPROM to unarmed
  /*!
      This sets the state of the EEPROM to unarmed
  */
  inline void unarm(void) { eestate = EEUNARMED; }
  
  //! Sets the EEPROM to not ready
  /*!
      This sets the state of the EEPROM to not ready
  */
  inline void unready(void) { eestate = EENOT_READY; }
  
  //! Sets the EEPROM to read
  /*!
      This sets the state of the EEPROM to read
  */
  inline void read(void) { eestate = EEREAD; }

  //! Indicates whether EEPROM is unarmed
  /*!
      \return True if unarmed, false otherwise
  */
  inline bool is_unarmed(void) { return (eestate == EEUNARMED); }
  
  //! Indicates whether EEPROM is not ready
  /*!
      \return True if not ready, false otherwise
  */
  inline bool is_not_ready(void) { return (eestate == EENOT_READY); }
  
  //! Indicates whether EEPROM is ready for writing
  /*!
      \return True if ready for write, false otherwise
  */
  inline bool is_ready_for_write(void) {
    return (eestate == EEREADY_FOR_WRITE);
  }

  //private:
  EEPROM *eeprom;          //! Pointer to associated EEPROM object
  enum EE_STATES eestate;  //! The state of the EEPROM
};

//
// EEDATA - EE data register
//

//! \class
/*!
    \brief This class models the EE data register
*/
class EEDATA : public SFR_Register
{
public:

  //! Default constructor
  /*!
      This creates an EEDATA object
  */
  EEDATA(void);

  //! Puts a new value into register
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  
  //! Gets the value of register
  /*!
      \return Value
  */
  unsigned int get(void);
  
  //! Sets the EEPROM reference of the EEDATA
  /*!
      \param ee Pointer to EEPROM
  */
  virtual void set_eeprom(EEPROM *ee) { eeprom = ee; }

  //private:
  EEPROM *eeprom;

};

//
// EEADR - EE address register
//

//! \class
/*!
    \brief This class models the EE address register
*/
class EEADR : public SFR_Register
{
public:

  //! Default constructor
  /*!
      Creates a new EEADR object
  */
  EEADR(void);

  //! Puts a new value into register
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  
  //! Gets the value of register
  /*!
      \return Value
  */
  unsigned int get(void);

  //! Sets the EEPROM reference of the EEADR
  /*!
      \param ee Pointer to EEPROM
  */
  virtual void set_eeprom(EEPROM *ee) { eeprom = ee; }

  //private:
  EEPROM *eeprom;
};


//------------------------------------------------------------------------
//------------------------------------------------------------------------

const int EPROM_WRITE_TIME = 20;

//! \class
/*!
    \brief This class models the EEPROM module. It contains the
    various control and register address required for EEPROM functionality.
*/
class EEPROM :  public BreakCallBack
{
public:

  //! Default constructor
  /*
      Creates a new EEPROM object
  */
  EEPROM(void);
  //! Resets the EEPROM
  /*!
      \param t The type of reset
  */
  void reset(RESET_TYPE t);
  
  //! Sets the CPU reference
  /*!
      \param p Pointer to Processor objec
  */
  void set_cpu(Processor *p) { cpu = p; }
  
  //! Sets the INTCON register reference
  /*!
      \param ic Pointer to INTCON object
  */
  void set_intcon(INTCON *ic);

  //! Notifies object that a break has occured
  void callback(void);
  
  //! Starts writing to EEPROM
  /*!
      This begins the writing of data to EEPROM
  */
  void start_write(void);
  
  //! Indicates that write is complete
  /*!
      This sets a flag to indicate that writing data to EEPROM 
      is complete.
  */
  void write_is_complete(void);
  
  void start_program_memory_read(void);  
  
  //! Initializes the EEPROM
  /*!
      This creates an array for the EEPROM and initializes the
      registers
      \param new_rom_size The size of EEPROM
  */
  void initialize(unsigned int new_rom_size);
  
  //! Gets a register in EEPROM
  /*!
      \param address The address of register
      \return Pointer to register
  */
  Register *get_register(unsigned int address);

  //! Changes the value of a register
  /*!
      \param offset Address of a register
      \param val Value to change to
  */
  inline void change_rom(unsigned int offset, unsigned int val) {
    assert(offset < rom_size);
    rom[offset]->value = val;
  }

  //! Gets the size of the EEPROM
  /*!
      \return Size of EEPROM
  */  
  inline unsigned int get_rom_size(void) { return (rom_size); }
  
  // XXX might want to make get_rom a friend only to cli_dump
  //! Gets the data area of EEPROM
  /*!
      \return EEPROM array of Registers
  */
  inline Register **get_rom(void) { return (rom); }

  //! Gets the EECON1 register
  /*!
      \return EECON1 object
  */
  inline EECON1 *get_reg_eecon1(void) { return (&eecon1); }
  //! Gets the EECON2 register
  /*!
      \return EECON2 object
  */
  inline EECON2 *get_reg_eecon2(void) { return (&eecon2); }
  //! Gets the EEDATA register
  /*!
      \return EEDATA object
  */
  inline EEDATA *get_reg_eedata(void) { return (&eedata); }
  //! Gets the EEADR register
  /*!
      \return EEADR object
  */
  inline EEADR *get_reg_eeadr(void) { return (&eeadr); }

  //! Dumps the contents of EEPROM to screen
  /*!
      Dumps the contents of the EEPROM to the console
  */
  void dump(void);

  //protected:
  //! Name of EEPROM
  char *name_str; 
  //! Processor object         
  Processor *cpu;  
  //! Pointer to INTCON         
  INTCON *intcon;           

  //! EE control register 1
  EECON1 eecon1;       
  //! EE control register 2     
  EECON2 eecon2;     
  //! EE data register       
  EEDATA eedata;   
  //! EE address register         
  EEADR  eeadr;           

  //! The EEPROM data area
  Register **rom;      
  //! The size of the EEPROM    
  unsigned int rom_size;  
  //! Latched address for eewrites  
  unsigned int wr_adr;  
  //! Latched data for eewrites    
  unsigned int wr_data; 
  //! Latches read for eewrites   
  unsigned int rd_adr;     
  //! Break point number that's set during eewrites
  unsigned int abp;         

};

/*class EEPROM_PIR : public EEPROM
{
public:

  virtual void set_pir_set(PIR_SET *p);

  // the 16f628 eeprom is identical to the 16f84 eeprom except
  // for the size and the location of EEIF. The size is taken
  // care of when the '628 is constructed, the EEIF is taken
  // care of here:

  virtual void write_is_complete(void);

  //protected:
  PIR_SET *pir_set;


};


class EEPROM_WIDE : public EEPROM
{
public:

  virtual void start_write(void);
  virtual void callback(void);
  virtual void start_program_memory_read(void);
  virtual void initialize(unsigned int new_rom_size);

  inline virtual EEDATA *get_reg_eedatah(void) { return (&eedatah); }
  inline virtual EEADR *get_reg_eeadrh(void) { return (&eeadrh); }

  //protected:
  EEDATA eedatah;
  EEADR  eeadrh;
};*/


#endif /* EEPROM_H */
