#ifndef _HWARE_MAP
#define _HWARE_MAP

#include <iostream>
#include <string>

using std::string;
using std::cout;
using std::endl;

//! \class
/*!
 * \brief Hardware Mapper class interfaces with parallel port
 * This class maps the physical hardware of the PIC16F84A to the pins of
 * the parallel port. The emulation is performed on the computer and 
 * periodically (10ms) the new pin data is sent to the pseudo-chip (the
 * header block) via the parallel port. 
 */
class HardwareMapper
{ 
   private:
      //! Address of LPT0, LPT1 or LPT2
      unsigned int lpt_add;
      
   public:
      //! Constructor
      /*!
          Creates a HardwareMapper object and initializes pin maps.
      */
      HardwareMapper(Processor *p);

      //! Destructor 
      ~HardwareMapper();
     
      //Declares which pins are used for input and which for output
      //int setIOArray(const int* IOArray); 
      
      //int setIOPinDirection(const int PICpin, const int value);
      
      //! Sets the LPT data
      /*!
          Sets according to what user has selected from GUI
          \param buttonID Radio button ID
      */
      void setLptData(int buttonID); 
      
      //! Checks if pin mapping is valid
      /*
          This function checks if the pin mapping selected by the
          user is valid
          \return -1 is successful, -2 otherwise
      */
      int checkIOValidity();
     
      //! Sets the write mode depending upon which pins are going to be used for IO 
      /*!
          \param mode Selected mode
      */
      void setIOMode(const int mode); 
      
      //! Returns the current status (0 or 1) of the selected PIC pin
      /*!
          \param picPin The PIC pin to check
          \return Status of PIC pin
      */
      int readSingle(const int picPin);

      //! Returns an array containing the status of all the pic pins
      /*!
          \return Array of pic pin status
      */
      int* readArray();

      //OVERLOADED FUNCTION
      //! Returns the status - high or low (1 or 0) - for an array of pins
      /*!
          \param picPins Array of PIC pins
          \param numPins No. of pins
          \return The status array
      */
      int* readArray(const int picPins[], const int numPins);

      
      //! Pass in full array of PIC pins and sets their values
      /*!
          \param picPins Array of PIC pins
          \return True if successful, false otherwise
      */
      bool setPins(const int picPins[]);
         
      //! Sends the content of the array via the LPT to the header block
      /*
          \return True if successful, false otherwise
      */
      bool write();
      //! Reads the contents of the LPT to array
      /*
          \return True if successful, false otherwise
      */
      bool read();

      //! Returns the LPT pin (0 - 24) currently assigned to picPin.
      /*
          \param picPin The PIC pin
          \return The assigned parallel port pin
      */
      int getPortPin(const int picPin);

      //! Returns the PIC pin (0 - 15) currently assigned to portPin.
      /*!
          \param portPin Parallel port pin
          \return The associated PIC pin
      */
      int getPicPin(const int portPin);

      //! Assigns a parallel port pin to a PIC pin.
      /*!
          \param picPin PIC pin
          \param portPin Parallel port pin
          \return True if successful, false otherwise
      */
      bool mapPin(const int picPin, const int portPin);
      
      //! Saves the current pin assignments to a file
      /*!
          \param filename The name of file
          \return True if successful, false otherwise
      */
      bool saveMap(const string filename);

      //! Loads the pin assignments from file
      /*!
          \param filename The name of file
          \return True if successful, false otherwise
      */
      bool loadMap(const string filename);
      
      //! Returns the direction (0=out/1=in) of the given picPin
      /*!
          \param picPin PIC pin
          \return IO status of pin (0=out/1=in)
      */
      int getIOStatus(const int picPin);

      //TESTING FUNCTIONS
      //! Prints the Pin Map
      void printPinMap();
      //! Prints the IO Status'
      void printIOStatus();
      //! Gets the Write Mode
      int getWriteMode();

   private:
      //! The number of pins on the PIC16F84A excluding VCC and GND
      static const int PIC_PINS = 18;

      //! The number of pins on the parallel port excluding GND
      static const int LPT_PINS = 17;
      
      //! Stores whether the pin is currently being used as IN or OUT
      int pinIOStatus[PIC_PINS];
      
      //! Stores the current value of the PIC pins.
      int currPinValue[PIC_PINS];
      
      //! Stores the previous value of PIC pin. Pushed from currPinValue[]
      int prevPinValue[PIC_PINS];
      
      //! The old value of Port A
      unsigned int oldPortA;
      //! The old value of Port B
      unsigned int oldPortB;
      
      //! Represents each LPT pin. Has value of corresponding PIC pin, or -1 if not assigned
      int portPinMap[LPT_PINS];
     
      //! Represents each PIC pin. Has value of corresponding LPT pin, or -1 if not assigned 
      int picPinMap[PIC_PINS];

      //! The write mode
      int writeMode;
      
      //! Pointer to Processor object
      Processor *cpu;
      
      //! The array of port pins
      int ports[18]; 
      
      //! Checks if the previous value of the pin had changed (low->high)
      /*!
          \param picPin PIC pin
          \return True if rising edge, false otherwise
      */
      bool risingEdge(const int picPin);

      //! Checks if the previous value of the pin had changed (high->low)
      /*!
          \param picPin PIC pin
          \return True if falling edge, false otherwise
      */
      bool fallingEdge(const int picPin);
};

#endif
