#ifndef INSTRUCTION_H
#define INSTRUCTION_H

#include <cstring>
#include <stdio.h>
//#include "processor.h"
//#include "register.h"

class Processor;
class Register;
class File_Register;

//! \class
/*!
    \brief This class models an instruction of the PIC16F84A
*/
class Instruction
{
   public:

  //! \enum 
  /*!
      \brief The various types of instruction
  */
  enum INSTRUCTION_TYPES
  {
    //! Normal instruction
    NORMAL_INSTRUCTION,
    //! Invalid instruction
    INVALID_INSTRUCTION,
    //! Breakpoint instruction
    BREAKPOINT_INSTRUCTION,
    //! Notify Instruction
    NOTIFY_INSTRUCTION,
    //! Profile Start Instruction
    PROFILE_START_INSTRUCTION,
    //! Profile Stop Instruction
    PROFILE_STOP_INSTRUCTION,
    //! Multiword Instruction
    MULTIWORD_INSTRUCTION
  };

  /*
   * Not all Instructions derived from the Instruction
   * class use these constants... 
   */

  enum
  {
   
    REG_MASK = 0x7f,
    DESTINATION_MASK = 0x80
    
  };
      //! Instruction name
      char name_str[20];
      //! Instruction opcode
      unsigned int opcode;
      //! Pointer to processor
      Processor* cpu;
      
      //! The source file that declared this instruction
      int file_id;   
      //! The hll source file that declared this instruction   
      int hll_file_id;    
      //! The line number within the source file 
      int src_line;    
      //! The line number within the list file       
      int lst_line;   
      //! The line number within the HLL source file       
      int hll_src_line;       

      //! Default constructor
      /*!
          Creates a new Instruction object
      */
      Instruction();
      //! Constructor 
      /*!
          Creates a new Instruction object
          \param proc Processor
          \param opc Opcode
          \param nameStr Name of instruction
      */
      Instruction(Processor* proc,unsigned int opc,char* nameStr);
      //! Returns the name of the instruction
      /*!
          \param return_str Buffer to store name into
          \return C String of name
      */
      virtual char* name(char* return_str) 
      {
         sprintf(return_str,"%s",name_str);
         return return_str;
      }; 
      //! Gets the opcode of the instruction
      /*!
          \return Opcode of instruction
      */
      unsigned int get_opcode() {return opcode;};
      
      //! Sets the opcode of the instruction
      /*!
          \param value New opcode
      */
      void set_opcode(int value) {opcode = value;};

      //! Executes the instruction
      virtual void execute(void){ };
      virtual void debug(void){ };
      //! Returns the instruction size
      /*!
          \return Instruction size (1)
      */
      virtual int Instruction_size(void) { return 1;};
      //! Returns the type of instruction
      /*!
          \return Instruction type (NORMAL_INSTRUCTION)
      */
      virtual INSTRUCTION_TYPES isa(void) {return NORMAL_INSTRUCTION;};
      
      //! Gets the src line associated with instruction
      /*!
          \return Source line number
      */
      virtual int get_src_line(void) { return(src_line); };
      
      //! Gets the HLL souce line
      /*!
          \return HLL source line number
      */
      virtual int get_hll_src_line(void) { return(hll_src_line); };
      //! Gets the LST file line
      /*!
          \return The LST file line number
      */
      virtual int get_lst_line(void) { return(lst_line); };
      //! Gets the file ID of source
      /*!
          \return The file ID
      */
      virtual int get_file_id(void) {return(file_id); };
      //! Gets the file ID of HLL source
      /*!
          \return The file ID
      */
      virtual int get_hll_file_id(void) {return(hll_file_id); };
      
      //! Decodes the instruction
      /*!
          \param new_cpu Processor object
          \param new_opcode New opcode
      */
      void decode(Processor *new_cpu, unsigned int new_opcode){cpu = new_cpu;opcode=new_opcode;};
      //! Adds line number symbol to address
      /*
          \param address Address
      */
      void add_line_number_symbol(int address);
      void update_line_number(int file, int sline, int lline, int hllfile, int hllsline);
      
      //! Initialize instruction
      /*! Some instructions require special initialization after they've
          been instantiated. For those that do, the instruction base class
          provides a way to control the initialization state (see the 16-bit
          PIC instructions).
      */
      virtual void initialize(bool init_state) {};
      
      //! Indicates whether instruction has been modified
      int is_modified;
      
};

//---------------------------------------------------------
//! \class
/*!
    \brief This class models an invalid instruction
*/
class Invalid_Instruction : public Instruction
{
public:

  //! Executes an invalid instruction (does nothing)
  virtual void execute(void);
  virtual void debug(void)
  {
    //cout << "*** INVALID INSTRUCTION ***\n";
  };

  //! Constructor for invalid instruction  
  /*!
      \param new_cpu Processor object
      \param new_opcode Opcode
  */
  Invalid_Instruction(Processor *new_cpu=NULL,unsigned int new_opcode=0);
  //! Returns type of instruction
  /*!
      \return Instruction type (INVALID_INSTRUCTION)
  */
  virtual INSTRUCTION_TYPES isa(void) {return INVALID_INSTRUCTION;};
  
  //! Name of instruction
  /*!
      \param return_str Buffer to store name
      \return C String of name
  */
  virtual char *name(char* return_str)
  {
     sprintf(return_str,"INVALID");
     return(return_str);
  };
  
  //! Static function to construct a new invalid instruction
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new Invalid_Instruction(new_cpu,new_opcode);}

};

extern Invalid_Instruction bad_instruction;

/*struct Instruction_Constructor
{
   unsigned int inst_mask;
   unsigned int opcode;
   char *name;
};*/

//-----------------------------------------------------------------
//
// instruction_constructor - a class used to create the PIC instructions
//
// The way it works is the 'instruction_constructor' structure
// contains three pieces of info for each instruction:
//   inst_mask - a bit mask indicating which bits uniquely
//               identify an instruction
//   opcode    - What those unique bits should be
//   inst_constructor - A pointer to the static member function
//                      'construct' in the instruction class.
//
// An instruction is decoded by finding a matching entry in
// the instruction_constructor array. A match is defined to
// be:
//    inst_mask & passed_opcode == opcode
// which means that the opcode that is passed to the decoder
// is ANDed with the instruction mask bits and compared to
// the base bits of the opcode. If this test passes, then the
// 'inst_constructor' will be called.

//! \struct
/*!
    \brief Instruction Constructor containg instruction mask, opcode
    and a pointer to the instruction
*/
struct Instruction_Constructor {
  //! Instruction mask
  unsigned int inst_mask;
  //! Instruction opcode
  unsigned int opcode;
  //! Pointer to Instruction
  Instruction * (*inst_constructor) (Processor *cpu, unsigned int inst);
};

//---------------------------------------------------------
//! \class
/*!
    \brief This class models a Literal Operand Instruction
*/
class Literal_op : public Instruction
{
public:
  //! Literal
  unsigned int L;

  //! Executes the instruction
  virtual void execute(void){ };
  //! Debug
  virtual void debug(void){ };
  //! Returns the name of instruction
  //! Name of instruction
  /*!
      \param return_str Buffer to store name
      \return C String of name
  */ 
  virtual char *name(char* return_str);

  //! Decodes the instruction
  /*
      \param new_cpu Processor object
      \param new_opcode Opcode
  */
  void decode(Processor *new_cpu, unsigned int new_opcode)
    {
      opcode = new_opcode;
      cpu = new_cpu;
      L = opcode & 0xff;
    }
};

//---------------------------------------------------------
//! \class
/*!
    \brief This class models Bit operand Instructions
*/
class Bit_op : public Instruction
{
public:
  //! Mask
  unsigned int mask;
  //! Register address
  unsigned int register_address;
  //! Indicates access
  bool access;
  //! Reference to register
  Register *reg;

  
  //! Executes the instruction
  virtual void execute(void){ };
  virtual void debug(void){ };
  //! Returns the name of instruction
  //! Name of instruction
  /*!
      \param return_str Buffer to store name
      \return C String of name
  */ 
  virtual char *name(char* return_str);

  //! Decodes the instruction
  /*
      \param new_cpu Processor object
      \param new_opcode Opcode
  */
  void decode(Processor *new_cpu, unsigned int new_opcode);

};


//---------------------------------------------------------
class Register_op : public Instruction
{
public:

  //! The source register
  static Register *source;
  //! Address of register
  unsigned int register_address;
  //! Destination
  bool destination;
  //! Access
  bool access;

  /*  Register *destination;*/
  //! Executes instruction
  virtual void execute(void){ };
  virtual void debug(void){ };
  //! Returns the name of instruction
  //! Name of instruction
  /*!
      \param return_str Buffer to store name
      \return C String of name
  */
  virtual char *name(char* return_str);

  //! Decodes the instruction
  /*
      \param new_cpu Processor object
      \param new_opcode Opcode
  */
  void decode(Processor *new_cpu, unsigned int new_opcode);

};

//---------------------------------------------------------

class ADDLW : public Literal_op
{

public:
  ADDLW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new ADDLW(new_cpu,new_opcode);}

};



//---------------------------------------------------------
class RETFIE : public Instruction
{
public:

  RETFIE(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new RETFIE(new_cpu,new_opcode);}

};


//---------------------------------------------------------
class RETURN : public Instruction
{
public:

  RETURN(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new RETURN(new_cpu,new_opcode);}

};

//---------------------------------------------------------

class SUBLW : public Literal_op
{

public:

  SUBLW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new SUBLW(new_cpu,new_opcode);}

};







//---------------------------------------------------------
class ADDWF : public Register_op
{
public:

  ADDWF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new ADDWF(new_cpu,new_opcode);}
};

//---------------------------------------------------------

class ANDLW : public Literal_op
{

public:
  ANDLW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new ANDLW(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class ANDWF : public Register_op
{
public:

  ANDWF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new ANDWF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class BCF : public Bit_op
{
public:

  BCF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);

  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new BCF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class BSF : public Bit_op
{
public:

  BSF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);

  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new BSF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class BTFSC : public Bit_op
{
public:

  BTFSC(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);

  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new BTFSC(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class BTFSS : public Bit_op
{
public:

  BTFSS(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);

  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new BTFSS(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class CALL : public Instruction
{
public:
  unsigned int destination;

  CALL(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  char *name(char *str);

  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new CALL(new_cpu,new_opcode);}
};

//---------------------------------------------------------
class CLRF : public Register_op
{
public:

  CLRF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  char *name(char *str);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new CLRF(new_cpu,new_opcode);}
};

//---------------------------------------------------------
class CLRW : public Instruction
{
public:

  CLRW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new CLRW(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class CLRWDT : public Instruction
{
public:

  CLRWDT(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new CLRWDT(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class COMF : public Register_op
{
public:

  COMF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new COMF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class DECF : public Register_op
{
public:

  DECF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new DECF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class DECFSZ : public Register_op
{
public:

  DECFSZ(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new DECFSZ(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class GOTO : public Instruction
{
public:
  unsigned int destination;

  GOTO(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  char *name(char *str);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new GOTO(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class INCF : public Register_op
{
public:

  INCF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new INCF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class INCFSZ : public Register_op
{
public:

  INCFSZ(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new INCFSZ(new_cpu,new_opcode);}

};

//---------------------------------------------------------

class IORLW : public Literal_op
{

public:
  IORLW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new IORLW(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class IORWF : public Register_op
{
public:

  IORWF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new IORWF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class MOVF : public Register_op
{
public:

  MOVF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new MOVF(new_cpu,new_opcode);}

};

//---------------------------------------------------------

class MOVLW : public Literal_op
{
public:
  MOVLW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new MOVLW(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class MOVWF : public Register_op
{
public:

  MOVWF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  char *name(char *str);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new MOVWF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class NOP : public Instruction
{
public:

  NOP(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new NOP(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class OPTION : public Instruction
{
public:

  OPTION(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new OPTION(new_cpu,new_opcode);}

};

//---------------------------------------------------------

class RETLW : public Literal_op
{
public:

  RETLW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new RETLW(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class RLF : public Register_op
{
public:

  RLF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new RLF(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class RRF : public Register_op
{
public:

  RRF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new RRF(new_cpu,new_opcode);}

};


//---------------------------------------------------------
class SLEEP : public Instruction
{
public:

  SLEEP(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new SLEEP(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class SUBWF : public Register_op
{
public:

  SUBWF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new SUBWF(new_cpu,new_opcode);}

};


//---------------------------------------------------------
class SWAPF : public Register_op
{
public:

  SWAPF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);

  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new SWAPF(new_cpu,new_opcode);}

};


//---------------------------------------------------------
class TRIS : public Register_op
{
public:
  File_Register *reg;

  TRIS(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  char *name(char *str);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new TRIS(new_cpu,new_opcode);}

};

//---------------------------------------------------------

class XORLW : public Literal_op
{

public:

  XORLW(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new XORLW(new_cpu,new_opcode);}

};

//---------------------------------------------------------
class XORWF : public Register_op
{
public:

  XORWF(Processor *new_cpu, unsigned int new_opcode);
  virtual void execute(void);
  static Instruction *construct(Processor *new_cpu, unsigned int new_opcode)
    {return new XORWF(new_cpu,new_opcode);}

};

#endif
      
   


