/*
   Copyright (C) 1998 T. Scott Dattalo

This file is part of gpsim.

gpsim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

gpsim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with gpsim; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#ifndef __INTERFACE_H__
#define __INTERFACE_H__

/*
 * interface.h
 *
 * Here are (hopefully) all of the definitions needed for an
 * external program (such as a gui) to interface with gpsim
 *
 */
#include <stdio.h>
#include <unistd.h>
//#include "modules.h"
#include "symbol.h"
#include <string>
//#include "breakpoints.h"

class Processor;

//! \enum
/*!
    \brief The types of register (RAM,EEPROM)
*/
typedef enum _REGISTER_TYPE
{
    //! RAM Register
    REGISTER_RAM,
    //! EEPROM Register
    REGISTER_EEPROM
} REGISTER_TYPE;

//! \struct
/*!
    \brief File context used for file handling
*/
struct file_context {
  //! Filename
  char *name;
  //! File pointer          
  FILE *file_ptr;
  //! An array of offsets into the file that point to the start of the source lines. 
  int *line_seek;      
  //! The maximum line number
  int max_line;
};

/*extern int verbose;
extern unsigned int microsim_is_initialized;
void  initialization_is_complete(void);
*/

#define INVALID_VALUE 0xffffffff

  /*********************************************************************
   * 
   * What follows are a whole bunch of functions that provide access to
   * gpsim's innards. This is what the gui uses to interface to gpsim.
   * All of the functions here have "C" signatures, which is to say
   * they may be called from "C" code as well as from C++
   */


#define SYMBOL_NAME_LEN 32

//! \struct
/*!
    \brief This struct used for symbols
*/
typedef struct _sym
{
    //! Symbol type
    enum SYMBOL_TYPE type;
    //! Symbol name
    char *name;
    //! Symbol value
    int value;
} sym;


//! \class
/*!
    \brief This class provides the GUI with an interface to the Processor 
*/
class Interface
{
   public:
      //! Processor object
      Processor* pic;
      
      //! Interface constructor
      /*!
          \param picPtr Processor pointer 
      */
      Interface(Processor* picPtr) { pic = picPtr; };
      
      //! Opens a source file
      /*!
          \param file Filename
          \return 1 if successful, 0 otherwise
      */
      int open(const char *file);
      
      //! Returns the program memory size
      /*!
          \return Program memory size
      */
      unsigned int get_program_memory_size();
      
      //! Gets the opcode 
      /*!
          \param address Address in program memory
          \return Opcode
      */
      unsigned int get_opcode(unsigned int address);
      
      //! Gets opcode name
      /*!
          \param address Address in program memory
          \param buffer Buffer to store name
          \return Name of opcode
      */
      char* get_opcode_name(unsigned int address,char* buffer);
      
      //! Puts a new opcode into address
      /*!
          \param address Address in program memory
          \param opcode New opcode
      */
      void put_opcode(unsigned int address,unsigned int opcode);
      
      //! Gets the status register value
      /*!
          \return Status value
      */
      unsigned int get_status();
      
      //! Gets the W register value
      /*!
          \return W reg value
      */
      unsigned int get_w();
      
      //! Gets the PC value
      /*!
          \return PC value
      */
      unsigned int get_pc();
      
      //! Puts a value into W
      /*!
          \param w_value New W value
      */ 
      void put_w(unsigned int w_value);   
      //! Puts a value into PC
      /*!
          \param pc_value New PC value
      */
      void put_pc_value(unsigned int pc_value);
      //! Puts a value into status
      /*! 
          \param status_value New Status value
      */
      void put_status(unsigned int status_value);
      
      //! Gets a register from address
      /*!
          \param type Register type
          \param register_number Register address
          \return Register pointer
      */
      Register* get_register_ref(REGISTER_TYPE type, unsigned int register_number);
      
      //! Gets the register memory size
      /*!
          \param type Register type
          \return Size of register memory
      */
      unsigned int get_register_memory_size(REGISTER_TYPE type);
      
      //! Gets the name of a register
      /*!
          \param type Register type
          \param register_number Register address
          \return Name of register
      */
      char *get_register_name(REGISTER_TYPE type,unsigned int register_number);
      
      //! Gets the value of a register
      /*!
          \param type Register type
          \param register_number Register address
          \return Value of register
      */
      unsigned int get_register_value(REGISTER_TYPE type,unsigned int register_number);
      
      //! Puts a value into a register
      /*!
          \param type Register type
          \param register_number Register address
          \param register_value New value
      */
      void put_register_value(REGISTER_TYPE type,unsigned int register_number,
            unsigned int register_value);
      
      //! Gets the file context of file
      /*!
          \param file_id File ID
          \return File context struct
      */
      struct file_context * get_file_context(unsigned int file_id);
      
      //! Toggles break at a source line
      /*!
          \param file_id File ID
          \param line Source line
      */
      void toggle_break_at_line(unsigned int file_id,unsigned int line);
      
      //! Toggles break at address
      /*!
          \param address Address of register
      */
      void toggle_break_at_address(unsigned int address);
      
      //! Toggles read break at address
      /*!
          \param address Address of register
      */
      void toggle_read_break_at_address(unsigned int address);
      
      //! Toggles write break at address
      /*!
          \param address Address of register
      */
      void toggle_write_break_at_address(unsigned int address);
      
      //! Clears breakpoints of register
      /*!
          \param address Address of register
      */
      void clear_breakpoints_at_address(unsigned int address);
      
      //! Indicates whether a register has a breakpoint set
      /*!
          \param type Register type
          \param register_number Register address
          \return 1 if bp set, 0 otherwise
      */
      unsigned int reg_has_breakpoint(REGISTER_TYPE type,unsigned int register_number);
      
      //! Determines the closest address in program memory to the source line
      /*!
          \param file_id File ID
          \param line Source line
          \return Program memory address
      */
      unsigned int find_closest_address_to_line(unsigned int file_id,unsigned int line);
      
      //! Gets the source line of address in program memory
      /*!
          \param address Program memory address
      */
      unsigned int get_src_line(unsigned int address);  
           
      //! Performs 1 step through program                                                                 
      void step();
      //! Runs program continuously 
      void run();
      //! Stops program
      void stop();
};
 

/*
  //---------------------------------------------------------------------------
  // processor functions
  //---------------------------------------------------------------------------
  char *microsim_processor_get_name(unsigned int processor_id);
  unsigned int microsim_get_pc_value(unsigned int processor_id);
  void  microsim_put_pc_value(unsigned int processor_id, unsigned int pc_value);
  unsigned int microsim_get_status(unsigned int processor_id);
  void microsim_put_status(unsigned int processor_id, unsigned int status_value);
  unsigned int microsim_get_w(unsigned int processor_id);
  void microsim_put_w(unsigned int processor_id, unsigned int w_value);
  unsigned int microsim_get_cycles_lo(unsigned int processor_id);
  guint64  microsim_get_cycles(unsigned int processor_id);
  double microsim_get_inst_clock(unsigned int processor_id);
  guint64  microsim_get_update_rate(void);
  void     microsim_set_update_rate(guint64);
  void microsim_assign_pc_xref(unsigned int processor_id, gpointer xref);
  void microsim_assign_trace_xref(gpointer xref);
  void microsim_get_current_trace(guint64 *current_cycle, int *trace_index,
			       char *current_trace, int bufsize);
  void microsim_trace_dump_to_file(int number_of_instructions, FILE *f);
  void microsim_step(unsigned int processor_id, unsigned int steps);
  void microsim_step_over(unsigned int processor_id);
  void microsim_hll_step(unsigned int processor_id);
  void microsim_hll_step_over(unsigned int processor_id);
  void microsim_run(unsigned int processor_id);
  void microsim_stop(unsigned int processor_id);
  void microsim_reset(unsigned int processor_id);
  void microsim_finish(unsigned int processor_id);
  void microsim_run_to_address(unsigned int processor_id, unsigned int address);
  unsigned int microsim_get_stack_size(unsigned int processor_id);
  unsigned int microsim_get_stack_value(unsigned int processor_id, unsigned int address);
  guint64 microsim_get_current_time(void);


  //---------------------------------------------------------------------------
  // Profiling
  //---------------------------------------------------------------------------
  void microsim_enable_profiling(unsigned int processor_id);
  void microsim_disable_profiling(unsigned int processor_id);
  guint64 microsim_get_cycles_used(unsigned int processor_id, unsigned int address);
  guint64 microsim_get_register_read_accesses(unsigned int processor_id, REGISTER_TYPE type, unsigned int address);
  guint64 microsim_get_register_write_accesses(unsigned int processor_id, REGISTER_TYPE type, unsigned int address);


  //---------------------------------------------------------------------------
  // misc gpsim functions
  //---------------------------------------------------------------------------
  gpointer microsim_set_cyclic_break_point( unsigned int processor_id, 
				   void (*interface_callback)(gpointer), 
				   gpointer interface_callback_data,
				   guint64 cycle);
  gpointer microsim_set_cyclic_break_point2( void (*interface_callback)(gpointer), 
					  gpointer interface_callback_data,
					  guint64 cycle);
  void microsim_clear_break(gpointer b);
  int microsim_open(unsigned int processor_id, const char *file);
  unsigned int microsim_get_number_of_source_files(unsigned int processor_id);
  unsigned int microsim_get_hll_file_id(unsigned int processor_id,
			  	 unsigned int address);
  unsigned int microsim_get_file_id(unsigned int processor_id,
				 unsigned int address);
  struct file_context * microsim_get_file_context(unsigned int processor_id,
					       unsigned int file_id);
  char *microsim_get_version(char *dest, int max_len);
  int microsim_get_hll_mode(unsigned int processor_id);
  int microsim_set_hll_mode(unsigned int processor_id, int mode);
  void microsim_set_bulk_mode(int flag);
  
  //---------------------------------------------------------------------------
  // symbol functions
  //---------------------------------------------------------------------------
  void microsim_symbol_rewind(unsigned int processor_id);
  sym *microsim_symbol_iter(unsigned int processor_id); // NULL on end


  //---------------------------------------------------------------------------
  // RAM/EEPROM memory functions
  //---------------------------------------------------------------------------
  unsigned int microsim_get_register_memory_size(unsigned int processor_id,
					      REGISTER_TYPE type);
  char *microsim_get_register_name(unsigned int processor_id,
				REGISTER_TYPE type,
				unsigned int register_number);
  unsigned int microsim_get_register_value(unsigned int processor_id,
					REGISTER_TYPE type,
					unsigned int register_number);
  void  microsim_put_register_value(unsigned int processor_id,
				 REGISTER_TYPE type,
				 unsigned int register_number,
				 unsigned int register_value);
  unsigned int microsim_reg_has_breakpoint(unsigned int processor_id,
					REGISTER_TYPE type,
					unsigned int register_number);
  unsigned int microsim_set_log_name(unsigned int processor_id, const char *filename, int format);
  unsigned int microsim_reg_set_read_logging(unsigned int processor_id,
					  REGISTER_TYPE type,
					  unsigned int register_number);
  unsigned int microsim_reg_set_write_logging(unsigned int processor_id,
					   REGISTER_TYPE type,
					   unsigned int register_number);
  unsigned int microsim_reg_set_read_value_logging(unsigned int processor_id,
						REGISTER_TYPE type,
						unsigned int register_number,
						unsigned int value,
						unsigned int mask);
  unsigned int microsim_reg_set_write_value_logging(unsigned int processor_id,
						 REGISTER_TYPE type,
						 unsigned int register_number,
						 unsigned int value,
						 unsigned int mask);
  gboolean microsim_register_is_alias(unsigned int processor_id,
				   REGISTER_TYPE type,
				   unsigned int register_number);
  gboolean microsim_register_is_sfr(unsigned int processor_id,
				 REGISTER_TYPE type,
				 unsigned int register_number);
  gboolean microsim_register_is_valid(unsigned int processor_id,
				   REGISTER_TYPE type,
				   unsigned int register_number);
  void  microsim_clear_register_xref(unsigned int processor_id,
				  REGISTER_TYPE type,
				  unsigned int reg_number,
				  gpointer xref);
  void  microsim_assign_register_xref(unsigned int processor_id,
				   REGISTER_TYPE type,
				   unsigned int reg_number,
				   gpointer xref);

  
  //---------------------------------------------------------------------------
  // program memory functions
  //---------------------------------------------------------------------------
  void microsim_set_read_break_at_address(unsigned int processor_id,
					  unsigned int address);
  void microsim_set_write_break_at_address(unsigned int processor_id,
					  unsigned int address);
  void microsim_set_execute_break_at_address(unsigned int processor_id,
					  unsigned int address);
  void microsim_set_notify_point_at_address(unsigned int processor_id,
					 unsigned int address,
					 void (*cb)(gpointer),
					 gpointer data);
  void microsim_set_profile_start_at_address(unsigned int processor_id,
					  unsigned int address,
					  void (*cb)(gpointer),
					  gpointer data);
  void microsim_set_profile_stop_at_address(unsigned int processor_id,
					 unsigned int address,
					 void (*cb)(gpointer),
					 gpointer data);
  void microsim_clear_profile_start_at_address(unsigned int processor_id,
					    unsigned int address);
  void microsim_clear_profile_stop_at_address(unsigned int processor_id,
					   unsigned int address);
  void microsim_clear_breakpoints_at_address(unsigned int processor_id,
					  unsigned int address);
  void microsim_toggle_break_at_address(unsigned int processor_id,
				     unsigned int address);
  void microsim_toggle_break_at_line(unsigned int processor_id,
				  unsigned int file_id,
				  unsigned int line);
  unsigned int  microsim_find_closest_address_to_line(unsigned int processor_id,
						   unsigned int file_id,
						   unsigned int line);
  unsigned int microsim_get_opcode(unsigned int processor_id,
				unsigned int address);
  void microsim_put_opcode(unsigned int processor_id,
			unsigned int address,
			unsigned int opcode);
  unsigned int microsim_get_src_line(unsigned int processor_id,
				  unsigned int address);
*/
#endif /* __INTERFACE_H__ */
