

#ifndef __IOPORTS_H__
#define __IOPORTS_H__

#include "enum.h"
#include "register.h"
class IOPORT_TRIS;
//class IOPORT_LATCH;
//class USART_MODULE;

//---------------------------------------------------------
// IOPORT

//! \class
/*!
    \brief This class models an IOPort such as Port A and Port B
*/
class IOPORT : public SFR_Register
{
public:

#define IOPINS 8

  //IOPIN  **pins; // [IOPINS];

  //! A mask that for those ports that don't have all 8 io bits.
  unsigned int valid_iopins; 
  //! Internal latch  
  unsigned int internal_latch;
  //! Number of I/O pins attached to this port
  unsigned int num_iopins;     

  //! Puts a value into register
  /*!
      \param new_value New value
  */
  virtual void put(unsigned int new_value);
  //! Puts a value into register
  /*!
      \param new_value New value
  */
  void put_value(unsigned int new_value);
  //! Sets a bit in the register
  /*!
      \param bit_number Bit to set
      \param new_value True for 1 or false for 0
  */
  virtual void setbit(unsigned int bit_number, bool new_value);
  //! Sets a bit in the register
  /*!
      \param bit_number Bit to set
      \param new_value True for 1 or false for 0
  */
  virtual void setbit_value(unsigned int bit_number, bool new_value);
  //! Gets a bit in the register
  /*!
      \param bit_number Bit to get
      \return The value of bit 
  */
  int get_bit(unsigned int bit_number);
  //int get_bit_voltage(unsigned int bit_number);
  //! Gets value of register
  /*!
      \return The value
  */
  unsigned int get(void);
  //! Gets value of register
  /*!
      \return The value
  */
  virtual unsigned int get_value(void);
  //void attach_stimulus(stimulus *new_stim, unsigned int bit_position);
  //virtual int update_stimuli(void);
  //void attach_iopin(IOPIN * new_pin, unsigned int bit_position);
  //void attach_node(Stimulus_Node *new_node, unsigned int bit_position);
  virtual void trace_register_write(void);
  //! Changes the direction of port pin to input/ouput
  /*!
      \param bit_number Pin number
      \param new_direction True for output, False for input
  */
  virtual void change_pin_direction(unsigned int bit_number, bool new_direction);

  //! Constructor
  /*!
      Creates a new IOPort object
      \param _num_iopins Number of IO pins
  */
  IOPORT(unsigned int _num_iopins=8);
  //! Destructor
  ~IOPORT();

};

//---------------------------------------------------------
// IOPORT

//! \class
/*!
    \brief This class models a PIC IO port containing a TRIS
*/
class PIC_IOPORT : public IOPORT
{
public:

  //! IOPORT TRIS register
  IOPORT_TRIS * tris;
  //IOPORT_LATCH * latch;   // non-null on 18x parts only

  //int update_stimuli(void);
  //! Puts a value into register
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  //! Changes the direction of pins according to TRIS
  /*!
      \param new_tris New TRIS
  */
  void update_pin_directions(unsigned int new_tris);\
  //! Changes the direction of port pin to input/ouput
  /*!
      \param bit_number Pin number
      \param new_direction True for output, False for input
  */
  void change_pin_direction(unsigned int bit_number, bool new_direction);
   
  PIC_IOPORT(unsigned int _num_iopins=8);

};

//! \class
/*!
    \brief This class models the IOPort TRIS register
*/
class IOPORT_TRIS : public SFR_Register
{
public:
  //! The PIC IO port associated with this TRIS 
  PIC_IOPORT * port;

  //! A mask that for those ports that don't have all 8 io bits.
  unsigned int valid_iopins;   

  //! Puts a value into register
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  //! Puts a value into register
  /*!
      \param new_value New value
  */
  void put_value(unsigned int new_value);
  //! Sets a bit in the register
  /*!
      \param bit_number Bit to set
      \param new_value True for 1 or false for 0
  */
  virtual void setbit(unsigned int bit_number, bool new_value);
  //! Gets value of register
  /*!
      \return The value
  */
  unsigned int get(void);
  //! Default constructor
  IOPORT_TRIS(void);

};


//! \class
/*!
    \brief This class models PORTB of PIC16F84A
*/
class PORTB : public PIC_IOPORT
{
public:
#define intedg_MASK (1<<6)
#define rbpu_MASK   (1<<7)

  //! RBPU 
  unsigned int rbpu;
  //! Interrupt Edge
  unsigned int intedg;

  //! Default Constructor
  PORTB(void);

  //! RBPU Interrupt Edge Update
  void rbpu_intedg_update(unsigned int);
  //! Gets value of register
  /*!
      \return The value
  */
  unsigned int get(void);
  //! Sets a bit in the register
  /*!
      \param bit_number Bit to set
      \param new_value True for 1 or false for 0
  */
  void setbit(unsigned int bit_number, bool new_value);

  //! Resets PORTB
  virtual void reset(RESET_TYPE r);

};

//! \class
/*!
    \brief This class models PORTA of PIC16F84A
*/
class PORTA : public PIC_IOPORT
{
public:

  //! Default Constructor
  PORTA(void);
  //! Sets a bit in the register
  /*!
      \param bit_number Bit to set
      \param new_value True for 1 or false for 0
  */
  void setbit(unsigned int bit_number, bool new_value);

};


#endif  // __IOPORTS_H__
