#include "processor.h"
#include "hexutils.h"
#include "fopen-path.h"
#include "breakpoints.h"

#include <iostream>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include "eeprom.h"
#include ".ui/microsim.h"

using std::cout;
using std::endl;

int verbose = 0;
guint64 simulation_start_cycle;
Processor::Processor(MicroSim *mc) 
{
   bad_instruction = new Invalid_Instruction();
   pc = new Program_Counter();
   config_modes = create_ConfigMode();
   simulation_mode = STOPPED;
   
   //abortSig.connect((QObject *)mc, SLOT(programAlert()));
   microSim = mc;
   
   files = NULL;
}

void Processor::init_program_memory(unsigned int memory_size)
{
   if ((memory_size-1) & memory_size)
   {
     cout << "*** WARNING *** memory_size should be of the form 2^N\n";
     memory_size = (memory_size + ~memory_size) & MAX_PROGRAM_MEMORY;
     cout << "rounding up to memory_size = " << memory_size << '\n';
   }
    
   program_memory = (Instruction**) new char[sizeof(Instruction*)*memory_size];
   if(program_memory == NULL)
   {
      cout<<"ERROR: Out of memory for program space\n";
      exit(1);
   }
   
   for(unsigned int i=0;i<memory_size;i++)
      program_memory[i] = bad_instruction;
}

void Processor::init_program_memory(int address, int value)
{
   if(address == config_word_address())
   {
     cout << "** SETTING CONFIG address = 0x"<<hex<< address << "  value = 0x"<<value<<'\n';
     set_config_word(address, value);
   }
   else
   {
      if((unsigned int)address < program_memory_size())
      {
         program_memory[address] = disasm(this,value);
         if(program_memory[address] == NULL)
            program_memory[address] = bad_instruction;
      }
      else if( (address>= 0x2100) && (address < 0x2100 + get_eeprom()->get_rom_size()))
      {    
         get_eeprom()->change_rom(address - 0x2100, value);
      }  
      pc->memory_size_mask = program_memory_size()- 1;
      pma.init(this);
   }
}

void Processor::build_program_memory(int *memory,int minaddr,int maxaddr)
{
   for(int i=minaddr;i<=maxaddr;i++)
      if((unsigned int)memory[i] != 0xffffffff)
         init_program_memory(i,memory[i]);
}

void Processor::load_hex(const char* hex_file)
{
   FILE* inputFile = fopen(hex_file,"r");
   if(inputFile == NULL)
   {
      cout<<"Error: Couldn't open "<<hex_file<<endl;
      return;
   }

   set_config_word(config_word_address(),0xffff);  // assume no configuration word is in the hex file.

   if (!readihex16 (this, inputFile))
   {
     // No errors were found in the hex file.

     fclose (inputFile);

     //build_program_memory(memory, minaddr, maxaddr);

     // Tell the gui that we've got some code.
     //gi.new_program(processor_id);

     //disassemble(minaddr, maxaddr);
     //if(verbose)
     //cout << "Configuration word = 0x"  << setw(4) << setfill('0') << get_config_word() << '\n';

     set_frequency(10e6);
     //set_config_word(::config_word);
     reset(POR_RESET);

     simulation_mode = STOPPED;

     //if(verbose)
     //cycles.dump_breakpoints();
    }

}

void Processor::listProgramMemory()
{
   unsigned int opcode;
   char* name;
   cout<<"Address:\tInstruction:\tOpcode:"<<endl;
   for(unsigned int i=0;i<program_memory_size();i++)
   {
      opcode = program_memory[i]->get_opcode();
      name = program_memory[i]->name_str;
      cout<<i<<"\t\t"<<name<<"\t\t"<<opcode<<endl;
   }
}

void Processor::listRegisters()
{
   for(unsigned int i=0;i<register_memory_size();i++)
   {
      cout<<i<<"\t\t"<<registers[i]->name()<<"\t\t"<<hex<<registers[i]->get_value()<<endl;
   }
}

//-------------------------------------------------------------------
//
// add_sfr_register
//
// The purpose of this routine is to add one special function register
// to the file registers. If the sfr has a physical address (like the 
// status or tmr0 registers) then a pointer to that register will be
// placed in the file register map. 

void Processor::add_sfr_register(SFR_Register *reg, unsigned int addr,
				      unsigned int por_value, char *new_name)
{

  reg->cpu         = this;
  if(addr < register_memory_size())
    {
      registers[addr] = reg;
      registers[addr]->address = addr;
      registers[addr]->alias_mask = 0;
      if(new_name)
        registers[addr]->new_name(new_name);

    }

  reg->value       = por_value;
  reg->por_value   = por_value;
  reg->wdtr_value  = por_value;
  reg->initialize();
}








//-------------------------------------------------------------------
//
// init_register_memory (unsigned int memory_size)
//
// Allocate an array for holding register objects.
//

void Processor::init_register_memory (unsigned int memory_size)
{

  /*if(verbose)
    cout << __FUNCTION__ << " memory size: " << memory_size << '\n';*/

  registers = (Register **) new char[sizeof (Register *) * memory_size];

  if (registers  == NULL)
    {
      cout << "*** ERROR *** Out of memory - PIC register space\n";
      exit (1);
    }


  // For processors with banked memory, the register_bank cooresponds to the
  // active bank. Let this point to the beginning of the register array for
  // now.

  register_bank = registers;

//RMA - Register Memory Access stuff
  rma.set_cpu(this);
  rma.set_Registers(registers, memory_size);
  
  // Make all of the file registers 'undefined' (each processor derived from this base
  // class defines its own register mapping).

  for (unsigned int i = 0; i < memory_size; i++)
    registers[i] = NULL;


}

//-------------------------------------------------------------------
//
//
// create_invalid_registers
//
//   The purpose of this function is to complete the initialization
// of the file register memory by placing an instance of an 'invalid
// file register' at each 'invalid' memory location. Most of PIC's
// do not use the entire address space available, so this routine
// fills the voids.
//

void Processor::create_invalid_registers (void)
{
  unsigned int i;

  /*if(verbose)
    cout << "Creating invalid registers " << register_memory_size()<<"\n";*/

  // Now, initialize any undefined register as an 'invalid register'
  // Note, each invalid register is given its own object. This enables
  // the simulation code to efficiently capture any invalid register
  // access. Furthermore, it's possible to set break points on
  // individual invalid file registers. By default, gpsim halts whenever
  // there is an invalid file register access.

  for (i = 0; i < register_memory_size(); i++)
    {
      if (NULL == registers[i])
      {
	  registers[i] = new Invalid_File_Register(i);
	  registers[i]->address = 0;    // BAD_REGISTER;
	  registers[i]->alias_mask = 0;
	  registers[i]->value = 0;	// unimplemented registers are read as 0
	  registers[i]->symbol_alias = NULL;

	  registers[i]->cpu = this;
	  //If we are linking with a gui, then initialize a cross referencing
	  //pointer. This pointer is used to keep track of which gui window(s)
	  //display the register. For invalid registers this should always be null.
	  //registers[i]->xref = NULL;

	}
    }  
}


//-------------------------------------------------------------------
//    add_file_registers
//
//  The purpose of this member function is to allocate memory for the
// general purpose registers.
//

void Processor::add_file_registers(unsigned int start_address, unsigned int end_address, unsigned int alias_offset)
{

  unsigned int j;

  // Initialize the General Purpose Registers:

  char str[100];
  for  (j = start_address; j <= end_address; j++) {

    registers[j] = new File_Register;
    if (alias_offset) {
      registers[j + alias_offset] = registers[j];
      registers[j]->alias_mask = alias_offset;
    } else
      registers[j]->alias_mask = 0;

    registers[j]->address = j;
    registers[j]->value = 0;
    //registers[j]->symbol_alias = NULL;

    //The default register name is simply its address
    sprintf (str, "0x%02x", j);
    registers[j]->new_name(str);

    registers[j]->cpu = this;

  }

}

//-------------------------------------------------------------------
//    delete_file_registers
//
//  The purpose of this member function is to delete file registers
//

void Processor::delete_file_registers(unsigned int start_address, unsigned int end_address)
{


  //  FIXME - this function is bogus.
  // The aliased registers do not need to be searched for - the alias mask
  // can tell at what addresses a register is aliased.

#define SMALLEST_ALIAS_DISTANCE  32
  unsigned int i,j;


  for (j = start_address; j <= end_address; j++) {
    if(registers[j]) {

      if(registers[j]->alias_mask) {
	// This registers appears in more than one place. Let's find all
	// of its aliases.
	for(i=SMALLEST_ALIAS_DISTANCE; i<register_memory_size(); i+=SMALLEST_ALIAS_DISTANCE)
	  if(registers[j] == registers[i])
	    registers[i] = NULL;
      }

      delete registers[j];
      registers[j] = NULL;
    }
  }

}

//-------------------------------------------------------------------
//
// 
//    alias_file_registers
//
//  The purpose of this member function is to alias the
// general purpose registers.
//

void Processor::alias_file_registers(unsigned int start_address, unsigned int end_address, unsigned int alias_offset)
{

  unsigned int j;

  // FIXME -- it'd probably make better sense to keep a list of register addresses at
  // which a particular register appears.

  for (j = start_address; j <= end_address; j++)
    {
      if (alias_offset)
	{
	  registers[j + alias_offset] = registers[j];
	  registers[j]->alias_mask = alias_offset;
	}
    }

}

//-------------------------------------------------------------------
//
// attach_src_line - This member function establishes the one-to-one link
// between instructions and the source code that create them.

void Processor::attach_src_line(int address,int file_id,int sline,int lst_line)
{
  if((unsigned int)address < program_memory_size())
    {
      //pma[address].update_line_number(file_id,sline,lst_line,0,0);
      program_memory[address]->update_line_number(file_id,sline,lst_line,0,0);
      //printf("%s address=%x, sline=%d, lst_line=%d\n", __FUNCTION__,address,sline,lst_line);
      if(sline > files[file_id].max_line)
	files[file_id].max_line = sline;

      // FIX ME - what is this '+5' junk?

      if(lst_line+5 > files[lst_file_id].max_line)
	files[lst_file_id].max_line = lst_line+5;

    }

}

//-------------------------------------------------------------------
// read_src_files - this routine will open all of the source files 
//   associated with the project and associate their line numbers
//   with the addresses of the opcodes they generated.
//

void Processor::read_src_files(void)
{
  // Are there any src files ?
  for(int i=0; i<number_of_source_files; i++) {


    // did this src file generate any code?
    if(files[i].max_line > 0) {
      
      // Create an array whose index corresponds to the
      // line number of a source file line and whose data
      // is the offset (in bytes) from the beginning of the 
      // file. (e.g. files[3].line_seek[20] references the
      // 20th line of the third source file.)
      files[i].line_seek = new int[files[i].max_line+1];
      if( NULL == (files[i].file_ptr = fopen_path(files[i].name,"r")))
	continue;
      rewind(files[i].file_ptr);

      char buf[256],*s;
      files[i].line_seek[0] = 0;
      for(int j=1; j<=files[i].max_line; j++)
	{
	  files[i].line_seek[j] = ftell(files[i].file_ptr);
	  s = fgets(buf,256,files[i].file_ptr);
	  if(s != buf)
	    break;
	}
    }
  }
}

void Processor::clear_src_files(void)
{
   delete files;
   files = NULL;
}


//-------------------------------------------------------------------
int ProgramMemoryAccess::find_closest_address_to_line(int file_id, int src_line)
{
  int closest_address = -1;

  if(!cpu)
    return closest_address;

  int distance = cpu->program_memory_size();

  for(int i = cpu->program_memory_size()-1; i>=0; i--) {

    // Find the closet instruction to the src file line number
    if( (cpu->program_memory[i]->isa() != Instruction::INVALID_INSTRUCTION) &&
	(cpu->program_memory[i]->get_file_id()==file_id)                    &&
	(abs(cpu->program_memory[i]->get_src_line() - src_line) < distance))

      {
	distance = abs(cpu->program_memory[i]->get_src_line() - src_line);
	closest_address = i;

	if(distance == 0)
        {
          //cout<<dec<<"Address: "<<cpu->map_pm_index2address(closest_address)<<" at line "<<src_line;
          //cout<<". src_line1="<<cpu->program_memory[i]->get_src_line()<<endl;
	  return cpu->map_pm_index2address(closest_address);
        }
      }
  }

  //cout<<dec<<"Address: "<<cpu->map_pm_index2address(closest_address)<<" at line "<<src_line;
  //cout<<". src_line2="<<cpu->program_memory[0]->get_src_line()<<endl;
  return cpu->map_pm_index2address(closest_address);

}
//--------------------------------------------------------------------------
//
// temporary - this could is going to get deleted as soon as file related 
// stuff gets put into its own object/class.

void ProgramMemoryAccess::set_hll_mode(int new_hll_mode)
{
  switch(new_hll_mode) {
  case ASM_MODE:
    hll_mode = ASM_MODE;
    break;
  case HLL_MODE:
    hll_mode = HLL_MODE;
  }
}
//--------------------------------------------------------------------------
unsigned int ProgramMemoryAccess::get_src_line(unsigned int address)
{
  unsigned int line = 0;
    
  if(!cpu || cpu->program_memory_size()<=address)
    return INVALID_VALUE;

  switch(get_hll_mode()) {

  case ASM_MODE:
    line = (this->operator[](address)).get_src_line();
    break;

  case HLL_MODE:
    line = (this->operator[](address)).get_hll_src_line();
    break;
  }

  // line 1 is first line (?), so zero or less means invalid line
  if(line<=0)
    return INVALID_VALUE;

  return line;
}

//--------------------------------------------------------------------------
unsigned int ProgramMemoryAccess::get_file_id(unsigned int address)
{
    
  if(!cpu)
    return INVALID_VALUE;

  switch(get_hll_mode()) {

  case ASM_MODE:
    return (this->operator[](address)).get_file_id();
    break;

  case HLL_MODE:
    return (this->operator[](address)).get_hll_file_id();
    break;
  }

  return INVALID_VALUE;

}


//-------------------------------------------------------------------
void ProgramMemoryAccess::set_break_at_address(int address)
{
  if(hasValid_opcode(address))
    bp.set_execution_break(cpu, address);

}

//-------------------------------------------------------------------
void ProgramMemoryAccess::set_read_break_at_address(int address)
{
    if(hasValid_opcode(address))
        bp.set_read_break(cpu, address);
}

//-------------------------------------------------------------------
void ProgramMemoryAccess::set_write_break_at_address(int address)
{
    if(hasValid_opcode(address))
        bp.set_write_break(cpu, address);
}

//-------------------------------------------------------------------
void ProgramMemoryAccess::set_notify_at_address(int address, BreakCallBack *cb)
{
  if(hasValid_opcode(address))
    bp.set_notify_break(cpu, address, cb);

}

//-------------------------------------------------------------------
void ProgramMemoryAccess::set_profile_start_at_address(int address, BreakCallBack *cb)
{
  int pm_index = cpu->map_pm_address2index(address);

  if( pm_index >= 0  && pm_index<cpu->program_memory_size()) 
    if (cpu->program_memory[pm_index]->isa() != Instruction::INVALID_INSTRUCTION)
      bp.set_profile_start_break(cpu, address, cb);
}

//-------------------------------------------------------------------
void ProgramMemoryAccess::set_profile_stop_at_address(int address, BreakCallBack *cb)
{
  if(hasValid_opcode(address))
    bp.set_profile_stop_break(cpu, address, cb);
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::clear_break_at_address(int address, 
				       enum Instruction::INSTRUCTION_TYPES type = 
				       Instruction::BREAKPOINT_INSTRUCTION)
{

  if( address >= 0  && address<cpu->program_memory_size()) {

    Instruction *instr = find_instruction(address,type);

    if(instr!=NULL) {
      int b = ((Breakpoint_Instruction *)instr)->bpn & BREAKPOINT_MASK;
      bp.clear( b );
      return 1;
    } 
  }

  return 0;
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::clear_notify_at_address(int address)
{
  return clear_break_at_address(address,Instruction::NOTIFY_INSTRUCTION);
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::clear_profile_start_at_address(int address)
{
  return clear_break_at_address(address,Instruction::PROFILE_START_INSTRUCTION);
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::clear_profile_stop_at_address(int address)
{
  return clear_break_at_address(address,Instruction::PROFILE_STOP_INSTRUCTION);
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::address_has_break(int address, 
					     enum Instruction::INSTRUCTION_TYPES type)
{
  if(find_instruction(address,type)!=NULL)
    return 1;

  return 0;
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::address_has_notify(int address)
{
  return address_has_break(address,Instruction::NOTIFY_INSTRUCTION);
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::address_has_profile_start(int address)
{
  return address_has_break(address,Instruction::PROFILE_START_INSTRUCTION);
}

//-------------------------------------------------------------------
int ProgramMemoryAccess::address_has_profile_stop(int address)
{
  return address_has_break(address,Instruction::PROFILE_STOP_INSTRUCTION);
}


//-------------------------------------------------------------------
void ProgramMemoryAccess::toggle_break_at_address(int address)
{
  if(address_has_break(address))
    clear_break_at_address(address);
  else
    set_break_at_address(address);
}

//-------------------------------------------------------------------
void ProgramMemoryAccess::toggle_read_break_at_address(int address)
{
  if(address_has_break(address))
    clear_break_at_address(address);
  else
    set_read_break_at_address(address);
}

//-------------------------------------------------------------------
void ProgramMemoryAccess::toggle_write_break_at_address(int address)
{
  if(address_has_break(address))
    clear_break_at_address(address);
  else
    set_write_break_at_address(address);
}


//-------------------------------------------------------------------

void ProgramMemoryAccess::set_break_at_line(int file_id, int src_line)
{
  int address;

  if( (address = find_closest_address_to_line(file_id, src_line)) >= 0)
      set_break_at_address(address);
}

void ProgramMemoryAccess::clear_break_at_line(int file_id, int src_line)
{

  int address;

  if( (address = find_closest_address_to_line(file_id, src_line)) >= 0)
      clear_break_at_address(address);
}

void ProgramMemoryAccess::toggle_break_at_line(int file_id, int src_line)
{
  toggle_break_at_address(find_closest_address_to_line(file_id, src_line));
}


void ProgramMemoryAccess::callback(void)
{

  if(_state)
    {
      _state = 0;
      //cout << __FUNCTION__ << " address= " << address << ", opcode= " << opcode << '\n';
      //cpu->program_memory[address]->opcode = opcode;
      put_opcode(_address,_opcode);
      //trace.opcode_write(_address,_opcode);
      bp.clear_pm_write();
    }

}


void Processor::run(int n)
{
  simulation_mode = RUNNING;
  int instructions = 0;
      
  //do parallel port stuff
  //cout<<"Write mode: "<<hm->getWriteMode()<<endl;
  if(hm->getWriteMode()==1 || hm->getWriteMode()==2)
  {
     //cout<<"Reading and writing\n";
     hm->read();
     hm->write();
  }
  do
  {
      do
      {
         program_memory[pc->value]->execute();
	 if(n > 1 && !bp.global_break && pma.hasValid_opcode(pc->value))
	     microSim->cacheInstruction(program_memory[pc->value]->name_str);
	 tmr0.internal_increment();
         instructions++;

      } while(simulation_mode != STOPPED && simulation_mode != ABNORMAL_STOP && !bp.global_break && instructions!=n);
      
     
      if(bp.have_interrupt())
         interrupt();

      if(bp.have_sleep())
         sleep();

      if(bp.have_pm_write())
         pm_write();
	 
    } while(simulation_mode != STOPPED && simulation_mode != ABNORMAL_STOP && !bp.global_break && instructions!=n);

   
  if(!bp.have_halt())
      bp.clear_global();
  else
      microSim->programHalt();
}



//-------------------------------------------------------------------
//
// run  -- Begin simulating and don't stop until there is a break.
//
void Processor::run (void)
{
  /*if(use_icd)
  {
      simulation_mode=RUNNING;
      icd_run();
      while(!icd_stopped())
      {
#ifdef HAVE_GUI
	  if(use_gui)
	      gui_refresh();
#endif
      }
      simulation_mode=STOPPED;
      disassemble(pc->get_value(), pc->get_value());
      gi.simulation_has_stopped();
      return;
  }*/


  if(simulation_mode != STOPPED) {
    /*if(verbose)*/
      cout << "Ignoring run request because simulation is not stopped\n";
    return;
  }

  simulation_mode = RUNNING;

  /*if(realtime_mode)
    realtime_cbp.start(active_cpu);*/

  // If the first instruction we're simulating is a break point, then ignore it.

  /*if(pma.find_instruction(pc->value,Instruction::BREAKPOINT_INSTRUCTION)!=NULL)
    {
      simulation_start_cycle = cycles.value;
    }*/

  /*MAIN LOOP THAT WE NEED TO REDO FOR OUR OWN STUFF*/  
  int instructions = 0;
      //int diff = 0;
      //int total_time;
      double dsec1, dus1;
      struct timeval start, end, a, b, c;
      struct timespec delay;
      gettimeofday(&start,NULL);
      gettimeofday(&a,NULL);
  do
    {
      
      
      //gettimeofday(&start,NULL);
      //gettimeofday(&a,NULL);
      do
      {
         program_memory[pc->value]->execute();
         tmr0.internal_increment();
         instructions++;
	
         
         if(instructions % 50000 == 0)
         {
            //do parallel port stuff
            //cout<<"Write mode: "<<hm->getWriteMode()<<endl;
            if(hm->getWriteMode()==1 || hm->getWriteMode()==2)
            {
               //cout<<"Reading and writing\n";
               hm->read();
               hm->write();
            }
	     
            gettimeofday(&b,NULL);
	     
            dsec1 = b.tv_sec - a.tv_sec;
            dus1 = b.tv_usec - a.tv_usec;
            cout << "Time for 50000 (usec) : "<<dsec1 * 1000000 + dus1 << std::endl;
            if((dsec1 * 1000000 + dus1) < 10000)
            {
               delay.tv_sec = 0;
               //cout<<"Diff (us) : "<<(double)diff<<endl;
                
               delay.tv_nsec = 8000000 - ((dsec1*1000000000) + (dus1*1000));
               cout<<"Proposed delay (us): "<<(double)(delay.tv_nsec/1000)<<endl;
               //delay only if need to.
               if((8000000 - ((dsec1*1000000000) + (dus1*1000)))>0)
                  nanosleep(&delay,NULL);
            }
            /*else
            {
               //maybe a time over 10000 variable and factor to this in to future delays
            }*/
             

            gettimeofday(&c,NULL);
            dsec1 = c.tv_sec - a.tv_sec;
            dus1 = c.tv_usec - a.tv_usec;
            /*we want to get diff to approach zero*/
            cout << "Time for 50000 (usec) delay : "<<dsec1 * 1000000 + dus1 << std::endl<<endl;
            gettimeofday(&a,NULL);
         }

      } while(simulation_mode != STOPPED && simulation_mode != ABNORMAL_STOP && !bp.global_break);
      
      
      /*if(simulation_mode == ABNORMAL_STOP)
	 microSim->programAlert();*/
      
      
      /*gettimeofday(&end,NULL);
      double dsec, dus;

      dsec = end.tv_sec - start.tv_sec;
      dus = end.tv_usec - start.tv_usec;
      cout << "Instructions : "<<(double)instructions<<endl;
      cout << "Time (usec) : "<<(double)(dsec * 1000000 + dus) << std::endl;
      cout << "Instructions per usec : "<<(double)instructions/(double)(dsec*1000000+dus)<<endl;*/
      
      
      

      if(bp.have_interrupt())
         interrupt();

      if(bp.have_sleep())
         sleep();

      if(bp.have_pm_write())
         pm_write();

    } while(simulation_mode != STOPPED && simulation_mode != ABNORMAL_STOP && !bp.global_break);

   gettimeofday(&end,NULL);
   double dsec, dus;

   dsec = end.tv_sec - start.tv_sec;
   dus = end.tv_usec - start.tv_usec;
   cout << "Instructions : "<<(double)instructions<<endl;
   cout << "Time (usec) : "<<(double)(dsec * 1000000 + dus) << std::endl;
   cout << "Instructions per usec : "<<(double)instructions/(double)(dsec*1000000+dus)<<endl;
  /*if(realtime_mode)
    realtime_cbp.stop();
*/ 
  if(!bp.have_halt())
      bp.clear_global();
  /*trace.cycle_counter(cycles.value);
  trace.dump_last_instruction();

  simulation_mode = STOPPED;*/

  /*gi.simulation_has_stopped();*/

}


//-------------------------------------------------------------------
//
// sleep - Begin sleeping and stay asleep until something causes a wake
//

void Processor::sleep (void)
{
  simulation_mode = SLEEPING;

  if(!bp.have_sleep())
    return;

  do
    {
      cycles.increment();   // burn cycles until something wakes us
      //cout<<"Getting stuck here"<<endl;
    } while(bp.have_sleep() && !bp.have_halt());

  if(!bp.have_sleep())
    pc->increment();

  simulation_mode = RUNNING;

}

void Processor::stop()
{
   if(simulation_mode!=ABNORMAL_STOP)
      simulation_mode = STOPPED;
   if(hm->getWriteMode()==1 || hm->getWriteMode()==2)
   {  
      hm->read();
      hm->write();
   }
}




//STEP ONE - run next instruction 
void Processor::step_one(void) {
    program_memory[pc->value]->execute();
    tmr0.internal_increment();
    if(hm->getWriteMode()==1 || hm->getWriteMode()==2)
    {  
       hm->read();
       hm->write();
    }
}


//-------------------------------------------------------------------
//
// step - Simulate one (or more) instructions. If a breakpoint is set
// at the current PC-> 'step' will go right through it. (That's supposed
// to be a feature.)
//
void Processor::step (unsigned int steps)
{
  if(simulation_mode != STOPPED) {
    if(verbose)
      cout << "Ignoring step request because simulation is not stopped\n";
    return;
  }

  simulation_mode = SINGLE_STEPPING;
  do
    {

      if(bp.have_sleep() || bp.have_pm_write())
	{
	  // If we are sleeping or writing to the program memory (18cxxx only)
	  // then step one cycle - but don't execute any code  

	  cycles.increment();
	  //trace.dump(1);

	}
      else if(bp.have_interrupt())
	{
          cout<<"bp.have_interrupt() is true"<<endl;
	  interrupt();
	}
      else
	{

	  step_one();
	  //trace.cycle_counter(cycles.value);
	  //trace.dump_last_instruction();

	} 

    }
  while(!bp.have_halt() && --steps>0);

  bp.clear_halt();
  simulation_mode = STOPPED;

  //gi.simulation_has_stopped();
}



/*STEP OVER - MUST MODIFY IT FOR OUR OWN WAY*/
//-------------------------------------------------------------------
//
// step_over - In most cases, step_over will simulate just one instruction.
// However, if the next instruction is a branching one (e.g. goto, call, 
// return, etc.) then a break point will be set after it and gpsim will
// begin 'running'. This is useful for stepping over time-consuming calls.
//

void Processor::step_over (void)
{

  unsigned int saved_pc = pc->value;

  if(simulation_mode != STOPPED) {
    if(verbose)
      cout << "Ignoring step-over request because simulation is not stopped\n";
    return;
  }

  step(1); // Try one step

  if( ! ( (pc->value >= saved_pc) && (pc->value <= saved_pc+2) ) )
    {
        if(pma.find_instruction(pc->value,Instruction::BREAKPOINT_INSTRUCTION)!=NULL)
	  return;
	else
	{
	  // Set a break point at the instruction just past the one over which we are stepping
	  unsigned int bp_num = bp.set_execution_break(this, saved_pc + 1);
	  run();
	  bp.clear(bp_num);
	}
    }

  // note that we don't need to tell the gui to update its windows since
  // that is already done by step() or run().

}

//-------------------------------------------------------------------
//
// step_over - In most cases, step_over will simulate just one instruction.
// However, if the next instruction is a branching one (e.g. goto, call, 
// return, etc.) then a break point will be set after it and gpsim will
// begin 'running'. This is useful for stepping over time-consuming calls.
//

/*void Processor::step_over (void)
{
  step(1); // Try one step
}
*/

/*HAVE TO FIND OUT WHAT THIS FUNCTION DOES - sleep??*/
void Processor::pm_write (void)
{
    simulation_mode = PM_WRITE;

  do
    {
      cycles.increment();     // burn cycles until we're through writing
    } while(bp.have_pm_write());

  simulation_mode = RUNNING;

}

//-------------------------------------------------------------------
void Processor::interrupt (void)
{

  bp.clear_interrupt();

  stack->push(pc->value);
  intcon->clear_gie();

  //trace.cycle_increment();

  pc->interrupt(INTERRUPT_VECTOR);

}




//-------------------------------------------------------------------
//
// reset - reset the pic based on the desired reset type.
//MUST CHANGE TO UNCOMMENT CERTAIN PARTS

void Processor::reset (RESET_TYPE r)
{

/*  if(use_icd)
  {
      puts("RESET");
      icd_reset();
      disassemble(pc->get_value(), pc->get_value());
      gi.simulation_has_stopped();
      return;
  }*/

  if(r == SOFT_RESET)
    {
     // trace.reset(r);
      pc->reset();
     // gi.simulation_has_stopped();
      cout << " --- Soft Reset (not fully implemented)\n";
      return;
    }

  //cout << " --- Reset\n";

  for(unsigned int i=0; i<register_memory_size(); i++)
    registers[i]->reset(r);


  clear_src_files();
  //trace.reset(r);
  pc->reset();
  stack->reset();
  W->reset(r);
  bp.clear_global();

  if(r == POR_RESET)
    {
      status->put_TO(1);
      status->put_PD(1);

      //if(verbose) {
	//cout << "POR\n";
//	if(config_modes) config_modes->print();
      //}
      if(config_modes)
      {
         cout << "Initialiased wdt with WDTE = "<<config_modes->get_wdt()<<" WDT timeout = "<<nominal_wdt_timeout<<endl;
         wdt.initialize( config_modes->get_wdt() , nominal_wdt_timeout);
         
         
      }
    }
  else if (r==WDT_RESET)
    status->put_TO(0);

  delete hm;
  hm = new HardwareMapper(this);
  //gi.simulation_has_stopped();
}




//-------------------------------------------------------------------
//
// disassemble - Disassemble the contents of program memory from
// 'start_address' to 'end_address'. The instruction at the current
// PC is marked with an arrow '==>'. If an instruction has a break
// point set on it then it will be marked with a 'B'. The instruction
// mnemonics come from the class declarations for each instruction.
// However, it is possible to modify this on a per instruction basis.
// In other words, each instruction in the program memory has it's
// own instantiation. So a MOVWF at address 0x20 is different than
// one at address 0x21. It is possible to change the mnemonic of
// one without affecting the other. As of version 0.0.7 though, this
// is not implemented.
//


//DO WE EVEN NEED THIS FUNCTION, IT JUST PRINTS TO STDOUT
void Processor::disassemble (int start_address, int end_address)
{
  Instruction *inst;
  //int use_src_to_disasm =0;

  if(start_address < 0) start_address = 0;
  if((unsigned int)end_address >= pc->memory_size_mask) end_address = pc->memory_size_mask;

  char str[50];

  for(int i = start_address; i<=end_address; i++)
    {
      str[0] =0;
      if (pc->value == (unsigned int)i)
	cout << "==>";
      else
	cout << "   ";
      inst = program_memory[i];

      // Breakpoints replace the program memory with an instruction that has
      // an opcode larger than 16 bits.

      if(program_memory[i]->opcode < 0x10000)
	{
	  cout << ' ';
	}
      //BREAKPOINT STUFF
      /*else
	{
	  cout << 'B';
	  Breakpoint_Instruction *bpi =  (Breakpoint_Instruction *)program_memory[i];
	  inst = bpi->replaced;
	}*/

      /*what the fuck does this do?? find which file from??*/
      /*if((files != NULL) && (use_src_to_disasm))
	{
	  int fid = program_memory[i]->file_id;
	  char buf[256];
	  //cout << " in " << files[fid].name << " @line " 
	  //     << program_memory[i]->src_line << '\n';
	  fseek(files[fid].file_ptr, 
		files[fid].line_seek[program_memory[i]->src_line - 1],
		SEEK_SET);
	  fgets(buf, 256, files[fid].file_ptr);
	  cout << buf;
	}
      else
	cout << hex << setw(4) << setfill('0') << i << "  "
	     << hex << setw(4) << setfill('0') << inst->opcode << "    "
	     << inst->name(str) << '\n';
      */
    }
}

//-------------------------------------------------------------------
//
// pic_processor -- list
//
// Display the contents of either a source or list file
//AGAIN DO WE NEED THIS FUNCTION - JUST TO STDOUT???
//
/*
void Processor::list(int file_id, int pc_val, int start_line, int end_line)
{


  if((number_of_source_files == 0) || (files == NULL))
    return;

  if(pc_val > program_memory_size())
    return;

  if(program_memory[pc_val]->isa() == Instruction::INVALID_INSTRUCTION)
    {
      cout << "There's no code at address 0x" << hex << pc_val << '\n';
      return;
    }

  int line,pc_line;
  if(file_id)
    {
      file_id = lst_file_id;
      line = program_memory[pc_val]->get_lst_line();
      pc_line = program_memory[pc->value]->get_lst_line();
    }
  else
    {
      file_id = program_memory[pc_val]->file_id;
      line = program_memory[pc_val]->get_src_line();
      pc_line = program_memory[pc->value]->get_src_line();
    }

  start_line += line;
  end_line += line;

  if(start_line < 0) start_line = 0;
  if(end_line > files[file_id].max_line)
    end_line = files[file_id].max_line;

  cout << " listing " << files[file_id].name << " Starting line " << start_line
       << " Ending line " << end_line << '\n';

  // Make sure that the file is open
  if(files[file_id].file_ptr == NULL)
    {
      if(files[file_id].name != NULL)
	files[file_id].file_ptr = fopen_path(files[file_id].name,"r");

      if(files[file_id].file_ptr == NULL)
	{
	  if(files[file_id].name != NULL)
	    cout << files[file_id].name;
	  else
	    cout << ".lst file";

	  cout  << " was not found\n";

	  return;
	}
    }

  for(int i=start_line; i<=end_line; i++)
    {
      char buf[256];
      fseek(files[file_id].file_ptr, 
	    files[file_id].line_seek[i],
	    SEEK_SET);
      fgets(buf, 256, files[file_id].file_ptr);

      if (pc_line == i)
	cout << "==>";
      else
	cout << "   ";

      cout << buf;
    }
}*/


//-------------------------------------------------------------------
//
// por - power on reset %%% FIX ME %%% needs lots of work...
//

void Processor::por(void)
{
  if(config_modes)
    wdt.initialize( config_modes->get_wdt(), nominal_wdt_timeout);

}

//-------------------------------------------------------------------
//
// 
//    create
//
//  The purpose of this member function is to 'create' a pic processor.
// Since this is a base class member function, only those things that
// are common to all pics are created.
//


//THIS NEEDS TO BE CHANGED TO HAVE AN APPROPRIATE CREATE FOR P16F84A

void Processor::create (void)
{

  init_program_memory (program_memory_size());

  init_register_memory (register_memory_size());

  //  create_invalid_registers (); THIS WAS COMMENTED OUT ORIGINALLY, LEAVE AS IS??
  //create_symbols(); WHEN SYMBOLS PART ADDED, UNCOMMENT THIS
  create_stack();

  // Now, initialize the core stuff:
  pc->cpu = this;

  wdt.cpu = this; //WHEN WDT IS DONE

  W = new WREG;
  W->cpu=this;
  pcl = new PCL;
  pclath = new PCLATH;
  status = new STATUS_Register;
  //FIXME more
  
  W->new_name("W");
  
  fsr = new FSR;
  indf = new INDF;
  fsr->new_name("FSR");

  register_bank = &registers[0];  // Define the active register bank 
  W->value = 0;
 
  //WHEN ADDED THESE PARTS, UNCOMMENT
  set_frequency(10e6);            // 
  nominal_wdt_timeout = 18e-3;    // 18ms according to the data sheet (no prescale)

  Vdd = 5.0;                      // Assume 5.0 volt power supply

  //trace.program_counter (pc->value);

  /*FOR ALL 14 BIT PROCESSORS ON CREATE*/
  tmr0.cpu = this;
  tmr0.start(0);

  //FOR ALL P16C8x PROCESSORS - SUPERCLASS OF P16F84
   
  create_iopin_map();
  
  
  EEPROM *e;
  e = new EEPROM;
  e->set_cpu(this);
  e->initialize(EEPROM_SIZE);
  e->set_intcon(&intcon_reg);
  set_eeprom(e);
  
  add_file_registers(0x0c, 0x4f, 0x80);
  
 
  //ADD THE SFR, UNCOMMENT ALL WHEN DOEN I.E TMR0, PORTA
  add_sfr_register(indf,   0x80);
  add_sfr_register(indf,   0x00);

  add_sfr_register(&tmr0,  0x01);
  add_sfr_register(&option_reg,  0x81, 0xff);

  add_sfr_register(pcl,    0x02, 0);
  add_sfr_register(status, 0x03, 0x18);
  add_sfr_register(fsr,     0x04);
  alias_file_registers(0x02,0x04,0x80);

  add_sfr_register(porta,   0x05);
  add_sfr_register(&trisa,  0x85, 0x3f);

  add_sfr_register(portb,   0x06);
  add_sfr_register(&trisb,  0x86, 0xff);

  add_sfr_register(get_eeprom()->get_reg_eedata(),  0x08);
  add_sfr_register(get_eeprom()->get_reg_eecon1(),  0x88, 0);

  add_sfr_register(get_eeprom()->get_reg_eeadr(),   0x09);
  add_sfr_register(get_eeprom()->get_reg_eecon2(),  0x89);

  add_sfr_register(pclath, 0x8a, 0);
  add_sfr_register(pclath, 0x0a, 0);

  add_sfr_register(&intcon_reg, 0x8b, 0);
  add_sfr_register(&intcon_reg, 0x0b, 0);
  
  intcon = &intcon_reg;

  create_invalid_registers();
//  p->pic_processor::create_symbols();
//symbol_table.add_module(p,p->name_str);

  hm = new HardwareMapper(this);
}

void Processor::create_iopin_map(void)
{

  // ---- This is probably going to be moved:
  porta = new PORTA;
  portb = new PORTB;

  // ---- Complete the initialization for the I/O Ports

  // Build the links between the I/O Ports and their tris registers.
  porta->tris = &trisa;
  trisa.port = porta;

  portb->tris = &trisb;
  trisb.port = portb;

  // And give them a more meaningful name.
  trisa.new_name("TRISA");

  trisb.new_name("TRISB");

  // Define the valid I/O pins.
  porta->valid_iopins = 0x1f;
  portb->valid_iopins = 0xff;
  
}


void Processor::set_eeprom(EEPROM *e)
{ 
  eeprom = e; 

  //  eeprom->cpu = this;

  ema.set_cpu(this);
  ema.set_Registers(e->rom, e->rom_size);

}

//-------------------------------------------------------------------

void Processor::set_config_word(unsigned int address,unsigned int cfg_word)
{
  config_word = cfg_word;

  // Clear all of the configuration bits in config_modes and then
  // reset each of them based on the config bits in cfg_word:
  //config_modes &= ~(CM_WDTE);
  //config_modes |= ( (cfg_word & WDTE) ? CM_WDTE : 0);
  //cout << " setting cfg_word and cfg_modes " << hex << config_word << "  " << config_modes << '\n';

  if((address == config_word_address()) && config_modes)
    config_modes->config_mode = (config_modes->config_mode & ~7) | (cfg_word & 7);

  if(verbose && config_modes)
    config_modes->print();

}

//-------------------------------------------------------------------
Instruction &ProgramMemoryAccess::operator [] (int address)
{
  //cout << "pma[0x"<< hex << address << "]\n";
  if(!cpu ||cpu->program_memory_size()<=(unsigned int)address  || address<0)
    return bad_instruction;

  return *cpu->program_memory[cpu->map_pm_address2index(address)];
}

//-------------------------------------------------------------------
Instruction *ProgramMemoryAccess::find_instruction(int address, enum Instruction::INSTRUCTION_TYPES type)
{

  if(cpu->program_memory_size()<=address  || address<0)
    return NULL;

  Instruction &q = this->operator[](address);
  if(q.isa()==Instruction::INVALID_INSTRUCTION)
    return NULL;


  Instruction *p = &q;

  for(;;)
    {
      if(p->isa()==type)
	return p;

      switch(p->isa())
	{
	case Instruction::MULTIWORD_INSTRUCTION:
	case Instruction::INVALID_INSTRUCTION:
	case Instruction::NORMAL_INSTRUCTION:
	  return NULL;
	case Instruction::BREAKPOINT_INSTRUCTION:
	case Instruction::NOTIFY_INSTRUCTION:
	case Instruction::PROFILE_START_INSTRUCTION:
	case Instruction::PROFILE_STOP_INSTRUCTION:
	  p=((Breakpoint_Instruction *)p)->replaced;
	  break;
	}

    }

  return NULL;
}


//-------------------------------------------------------------------
//-------------------------------------------------------------------
//

ProgramMemoryAccess::ProgramMemoryAccess(void)
{
  _address = 0;
  _opcode = 0;
  _state = 0;
  hll_mode = ASM_MODE;
}

void ProgramMemoryAccess::init(Processor *new_cpu)
{
  cpu = new_cpu;
  
}

void ProgramMemoryAccess::put(int address, Instruction *new_instruction)
{

  if(!new_instruction)
    return;


  if(hasValid_opcode(address)) {

    cpu->program_memory[cpu->map_pm_address2index(address)] = new_instruction;

    //if(new_instruction->xref)
      //new_instruction->xref->update();
  }

}

Instruction *ProgramMemoryAccess::get(int address)
{
  if((unsigned int)address < cpu->program_memory_size())
    return &(this->operator[](address));
  else
    return NULL;

}

// like get, but will ignore instruction break points 
Instruction *ProgramMemoryAccess::get_base_instruction(int address)
{
    Instruction *p;

    p=get(address);

    if(p==NULL)
        return NULL;

    for(;;)
    {
	switch(p->isa())
	{
	case Instruction::MULTIWORD_INSTRUCTION:
	case Instruction::INVALID_INSTRUCTION:
	case Instruction::NORMAL_INSTRUCTION:
            return p;
	case Instruction::BREAKPOINT_INSTRUCTION:
	case Instruction::NOTIFY_INSTRUCTION:
	case Instruction::PROFILE_START_INSTRUCTION:
	case Instruction::PROFILE_STOP_INSTRUCTION:
	    p=((Breakpoint_Instruction *)p)->replaced;
            break;
	}

    }
    return NULL;
}

//----------------------------------------
// get_opcode - return an opcode from program memory.
//              If the address is out of range return 0.

unsigned int ProgramMemoryAccess::get_opcode(int addr)
{

  if((unsigned int)addr < cpu->program_memory_size())
    return(cpu->program_memory[addr]->get_opcode());
  else
    return 0;
}


//----------------------------------------
// get_opcode_name - return an opcode name from program memory.
//                   If the address is out of range return NULL;

char *ProgramMemoryAccess::get_opcode_name(int addr, char* buffer, int size)
{

  if((unsigned int)addr < cpu->program_memory_size()) 
    return cpu->program_memory[addr]->name(buffer);

  return NULL;
}

//----------------------------------------
// Get the current value of the program counter.
unsigned int ProgramMemoryAccess::get_PC(void)
{
  return 0;
}

void ProgramMemoryAccess::put_opcode_start(int addr, unsigned int new_opcode)
{

  if( ((unsigned int)addr < cpu->program_memory_size()) && (_state == 0))
    {
      _state = 1;
      _address = addr;
      _opcode = new_opcode;
      cycles.set_break_delta(40000, this);
      bp.set_pm_write();
    }

}

void ProgramMemoryAccess::put_opcode(int addr, unsigned int new_opcode)
{
  int i;

  if( !(addr>=0) && ((unsigned int)addr < cpu->program_memory_size()))
    return;


  Instruction *old_inst = get_base_instruction(addr);
  Instruction *new_inst = cpu->disasm_cpu(addr,new_opcode);

  if(new_inst==NULL)
  {
      puts("FIXME, in ProgramMemoryAccess::put_opcode");
      return;
  }
  
  if(!old_inst) {
    put(addr,new_inst);
    return;
  }

  if(old_inst->isa() == Instruction::INVALID_INSTRUCTION) {
    put(addr,new_inst);
    return;
  }

  // Now we need to make sure that the instruction we are replacing is
  // not a multi-word instruction. The 12 and 14 bit cores don't have
  // multi-word instructions, but the 16 bit cores do. If we are replacing
  // the second word of a multiword instruction, then we only need to
  // 'uninitialize' it. 

  // if there was a breakpoint set at addr, save a pointer to the breakpoint.
  Breakpoint_Instruction *b=bpi;
  Instruction *prev = get_base_instruction(addr-1);

  if(prev)
    prev->initialize(false);

  new_inst->update_line_number(old_inst->get_file_id(), 
			       old_inst->get_src_line(), 
			       old_inst->get_lst_line(),
			       old_inst->get_hll_src_line(),
			       old_inst->get_hll_file_id());

  //new_inst->xref = old_inst->xref;

  if(b) 
    b->replaced = new_inst;
  else
    cpu->program_memory[addr] = new_inst;

  cpu->program_memory[addr]->is_modified=1;
  
  //if(cpu->program_memory[addr]->xref)
  //    cpu->program_memory[addr]->xref->update();
  
  delete(old_inst);
}

//--------------------------------------------------------------------------

/*void  ProgramMemoryAccess::assign_xref(unsigned int address, gpointer xref)
{

  instruction &q = this->operator[](address);
  if(q.isa()==Instruction::INVALID_INSTRUCTION)
    return;

  if(q.xref)
    q.xref->add(xref);
}*/

//--------------------------------------------------------------------------
void ProgramMemoryAccess::step(unsigned int steps)
{

  if(!cpu)
    return;

  switch(get_hll_mode()) {

  case ASM_MODE:
    cpu->step(steps);
    break;

  case HLL_MODE:
    {
      unsigned int initial_line = cpu->pma.get_src_line(cpu->pc->get_value());
      unsigned int initial_pc = cpu->pc->get_value();

      while(1)
	{
	  cpu->step(1);

	  if(cpu->pc->get_value()==initial_pc)
	    break;

	  if(get_src_line(cpu->pc->get_value())
	     != initial_line)
	    break;
	}

      break;
    }
  }
}

//--------------------------------------------------------------------------
void ProgramMemoryAccess::step_over(void)
{

  if(!cpu)
    return;

  switch(get_hll_mode()) {

  case ASM_MODE:
    cpu->step_over();
    break;

  case HLL_MODE:
    {

#if 0
      //
      // High level language step-overs are only supported for 
      // pic processors
      //

      pic_processor *pic = dynamic_cast<pic_processor *>cpu;

      if(!pic) {
	cout << "step-over is not supported for non-PIC processors\n";
	return;
      }
      unsigned int initial_line = cpu->pma.get_src_line(cpu->pc->get_value());
      unsigned int initial_pc = cpu->pc->get_value();
      unsigned int initial_stack_depth = pic->stack->pointer&pic->stack->stack_mask;

      while(1)
	{
	  step(1);

	  if(initial_stack_depth < pic->stack->pointer&pic->stack->stack_mask)
	    gpsim_finish(processor_id);

	  if(pic->pc->get_value()==initial_pc)
	    break;

	  if(get_src_line(initial_pc) != initial_line)
	    {
	      if(gpsim_get_file_id(processor_id, pic->pc->get_value()))
		break;
	    }
	}
      break;
#endif
      cout << "HLL Step is not supported\n";
    }
  }

  //redisplay_prompt();
}

//--------------------------------------------------------------------------

bool  ProgramMemoryAccess::hasValid_opcode(unsigned int address)
{

  if((this->operator[](address)).isa() != Instruction::INVALID_INSTRUCTION)
    return true;

  return false;
}
//--------------------------------------------------------------------------

bool  ProgramMemoryAccess::isModified(unsigned int address)
{

  if((address < cpu->program_memory_size()) && 
     cpu->program_memory[address]->is_modified)
    return true;

  return false;
}


//========================================================================
// Register Memory Access 

RegisterMemoryAccess::RegisterMemoryAccess(void)
{
  cpu = NULL;
  registers = NULL;
  nRegisters = 0;
}

//--------------------------------------------------------------------------

Register *RegisterMemoryAccess::get_register(unsigned int address)
{

  if(!cpu || !registers || nRegisters<=address)
    return NULL;

  Register *reg = registers[address];

  // If there are breakpoints set on the register, then drill down
  // through them until we get to the real register.

  while(reg->isa() == Register::BP_REGISTER)
    reg = ((Notify_Register *)reg)->replaced;


  return reg;

}

//--------------------------------------------------------------------------
void RegisterMemoryAccess::set_cpu(Processor *p)
{ 
  cpu = p;
}


//--------------------------------------------------------------------------
void RegisterMemoryAccess::set_Registers(Register **_registers, int _nRegisters) 
{ 
  nRegisters = _nRegisters; 
  registers = _registers;
}

bool RegisterMemoryAccess::hasBreak(int address)
{

  if(!cpu || !registers || nRegisters<=address)
    return false;

  return registers[address]->isa() == Register::BP_REGISTER;

}

  





















//------------------------------------------------------------------------
// member functions for the Program_Counter base class
//------------------------------------------------------------------------
//

//--------------------------------------------------

Program_Counter::Program_Counter(void)
{
  /*if(verbose)   cpu->init_program_memory(0x3FF);
    cout << "pc constructor\n";*/

  reset_address = 0;
  value = 0;
  pclath_mask = 0x1800;    // valid pclath bits for branching in 14-bit cores 

  //xref = new XrefObject(&value);


}

//--------------------------------------------------
// increment - update the program counter. All non-branching instructions pass through here.
//   

void Program_Counter::increment(void)
{
  
  value = (value + 1) & memory_size_mask;
  //trace.program_counter(value);

  // Update pcl. Note that we don't want to pcl.put() because that 
  // will trigger a break point if there's one set on pcl. (A read/write
  // break point on pcl should not be triggered by advancing the program
  // counter).

  cpu->pcl->value = value & 0xff;
  cycles.increment();//this obviously has to do with the WDT
  
}

//--------------------------------------------------
// skip - Does the same thing that increment does, except that it records the operation
// in the trace buffer as a 'skip' instead of a 'pc update'.
//   

void Program_Counter::skip(void)
{

  //trace.cycle_increment();
  value = (value + 1) & memory_size_mask;
  //trace.pc_skip(value);


  // Update pcl. Note that we don't want to pcl.put() because that 
  // will trigger a break point if there's one set on pcl. (A read/write
  // break point on pcl should not be triggered by advancing the program
  // counter).

  cpu->pcl->value = value & 0xff;
  cycles.increment();
}

//--------------------------------------------------
// jump - update the program counter. All branching instructions except computed gotos
//        and returns go through here.

void Program_Counter::jump(unsigned int new_address)
{

  // Use the new_address and the cached pclath (or page select bits for 12 bit cores)
  // to generate the destination address:
  
  //value = (new_address | cpu_pic->get_pclath_branching_jump() ) & memory_size_mask;
  value = new_address & memory_size_mask;
  
    // see Update pcl comment in Program_Counter::increment()

  cpu->pcl->value = value & 0xff;
  
  cycles.increment();
  //trace.program_counter(value);
  cycles.increment();

}

//--------------------------------------------------
// interrupt - update the program counter. Like a jump, except pclath is ignored.
//

void Program_Counter::interrupt(unsigned int new_address)
{

  // Use the new_address and the cached pclath (or page select bits for 12 bit cores)
  // to generate the destination address:

  value = new_address & memory_size_mask;

  cpu->pcl->value = value & 0xff;    // see Update pcl comment in Program_Counter::increment()
  
  cycles.increment();
  
  //trace.cycle_increment(); 
  //trace.program_counter(value);

  cycles.increment();

}

//--------------------------------------------------
// computed_goto - update the program counter. Anytime the pcl register is written to
//                 by the source code we'll pass through here.
//

void Program_Counter::computed_goto(unsigned int new_address)
{

  // Use the new_address and the cached pclath (or page select bits for 12 bit cores)
  // to generate the destination address:

  value = (new_address | cpu->get_pclath_branching_modpcl() ) & memory_size_mask;

  //trace.cycle_increment();
  //trace.program_counter(value);

  // see Update pcl comment in Program_Counter::increment()

  cpu->pcl->value = value & 0xff;

  // The instruction modifying the PCL will also increment the program counter.
  // So, pre-compensate the increment with a decrement:
  value--;
  cycles.increment();
}

//--------------------------------------------------
// new_address - write a new value to the program counter. All returns pass through here.
//

void Program_Counter::new_address(unsigned int new_value)
{
  value = new_value & memory_size_mask;
  //trace.cycle_increment();
  //trace.program_counter(value);

  // see Update pcl comment in Program_Counter::increment()

  cpu->pcl->value = value & 0xff;
  cycles.increment();
  cycles.increment();
}

//--------------------------------------------------
// get_next - get the next address that is just pass the current one
//            (used by 'call' to obtain the return address)

unsigned int Program_Counter::get_next(void)
{

  return( (value + cpu->program_memory[value]->Instruction_size()) & memory_size_mask);
  

}


//--------------------------------------------------
// put_value - Change the program counter without affecting the cycle counter
//             (This is what's called if the user changes the pc.)

void Program_Counter::put_value(unsigned int new_value)
{
  // FIXME 
  #define PCLATH_MASK              0x1f

  value = new_value & memory_size_mask;
  cpu->pcl->value = value & 0xff;
  cpu->pclath->value = (new_value >> 8) & PCLATH_MASK;

  //trace.program_counter(value);
/*
  if(xref)
    {
      if(cpu_pic->pcl->xref)
	cpu_pic->pcl->xref->update();
      if(cpu_pic->pclath->xref)
	cpu_pic->pclath->xref->update();
	xref->update();
    }*/
  

}

void Program_Counter::reset(void)
{ 
  value = reset_address;
  //trace.program_counter(value);
}

//-------------------------------------------------------------------
//  ConfigMode
//
void ConfigMode::print(void)
{


  if(config_mode & CM_FOSC1x) {
    // Internal Oscillator type processor 

    switch(config_mode& (CM_FOSC0 | CM_FOSC1)) {  // Lower two bits are the clock type
    case 0: cout << "LP"; break;
    case CM_FOSC0: cout << "XT"; break;
    case CM_FOSC1: cout << "Internal RC"; break;
    case (CM_FOSC0|CM_FOSC1): cout << "External RC"; break;

    } 
  }else {
    switch(config_mode& (CM_FOSC0 | CM_FOSC1)) {  // Lower two bits are the clock type
    case 0: cout << "LP"; break;
    case CM_FOSC0: cout << "XT"; break;
    case CM_FOSC1: cout << "HS"; break;
    case (CM_FOSC0|CM_FOSC1): cout << "RC"; break;
    }
  }

  cout << " oscillator\n";

  if(valid_bits & CM_WDTE) 
    cout << " WDT is " << (get_wdt() ? "enabled\n" : "disabled\n");

  if(valid_bits & CM_MCLRE) 
    cout << "MCLR is " << (get_mclre() ? "enabled\n" : "disabled\n");

  if(valid_bits & CM_CP0) {

    if(valid_bits & CM_CP1) {
      cout << "CP0 is " << (get_cp0() ? "high\n" : "low\n");
      cout << "CP1 is " << (get_cp1() ? "high\n" : "low\n");
    } else {

      cout << "code protection is " << (get_cp0() ? "enabled\n" : "disabled\n");

    }
  }


}


void Processor::set_frequency(double f) 
{ 
   frequency = f; 
   if(f>0) period = 1/f; 
}

unsigned int Processor::time_to_cycles( double t) 
{
   if(period>0) 
      return (int)(frequency * t); 
   else return 0;
}





   
