#ifndef PROCESSOR_H
#define PROCESSOR_H

#include "interface.h"
//#include "breakpoints.h"
#include "enum.h"
#include "cycles.h"
#include "tmr0.h"
#include "ioports.h"
#include "hardware_mapper.h"
#include "wdt.h"
//#include "eeprom.h"
#include <qsignal.h>

class MicroSim;
class Program_Counter;
class Register;
class SFR_Register;
class Processor;
class EEPROM;
class BreakCallBack;
class Breakpoint_Instruction;
extern int verbose;
extern Instruction *disasm(Processor *cpu,unsigned int inst);

//---------------------------------------------------------
// 
//

//! \class
/*!
    \brief This provides an interface to the program memory
    
    The ProgramMemoryAccess class is the interface used
    by objects other than the simulator to manipulate the 
    pic's program memory. For example, the breakpoint class
    modifies program memory when break points are set or 
    cleared. The modification goes through here.
*/
class ProgramMemoryAccess : public BreakCallBack
{
 public:
  //! Processor pointer
  Processor *cpu;

  //! Address 
  unsigned int _address;
  //! Opcode
  unsigned int _opcode;
  //! State
  unsigned int _state;

  //! Symbolic debugging
  enum HLL_MODES {
    //! Source came from plain old .asm files
    ASM_MODE,   
    //! Source came from a high level language like C or JAL.   
    HLL_MODE       
  };

  enum HLL_MODES hll_mode;

  /*! breakpoint instruction pointer. This is used by get_base_instruction().
      If an instruction has a breakpoint set on it, then get_base_instruction
      will return a pointer to the instruction and will initialize bpi to
      the breakpoint instruction that has replaced the one in the processor's
      program memory.
  */
  Breakpoint_Instruction *bpi;

  //! Overriden operator []
  /*!
      \param address Address in program memory
  */
  Instruction &operator [] (int address);

  //! Puts a new instruction in address
  /*
      \param addr Address
      \param new_instruction New instruction
  */
  void put(int addr, Instruction *new_instruction);
  
  //! Gets instruction
  /*!
      \param addr Address
      \return Instruction
  */
  Instruction *get(int addr);
  
  //! Gets base instruction
  /*!
      \param addr Address
      \return Instruction
  */
  Instruction *get_base_instruction(int addr);
  
  //! Gets opcode
  /*!
      \param addr Address
      \return Opcode
  */
  unsigned int get_opcode(int addr);
  
  //! Gets opcode name
  /*!
      \param addr Address
      \param buffer Buffer to store name
      \param size Size of buffer
      \return Opcode name
  */
  char *get_opcode_name(int addr, char* buffer, int size);
  
  //! Gets PIC
  /*!
      \return Program Counter
  */
  unsigned int get_PC(void);

  //! Puts an opcode in address
  /*!
      \param addr Address
      \param new_opcode
  */
  void put_opcode(int addr, unsigned int new_opcode);
  
  //! When a pic is replacing one of it's own instructions, this routine is called.
  /*!
      \param addr Address
      \param new_opcode New opcode
  */
  void put_opcode_start(int addr, unsigned int new_opcode);

  // Assign a cross reference object to an instruction 
  //void assign_xref(unsigned int address, gpointer cross_reference);

  //! Callback (BreakCallBack)
  void callback(void);
  
  //! Default Constructor
  ProgramMemoryAccess(void);
  
  //! Initializes the Program Memory Access
  void init(Processor *);
  
  // Helper functions for querying the program memory

  //! Determines if address has a valid opcode
  /*!
      /param address Address
      /return True if valid, false otherwise
  */
  bool hasValid_opcode(unsigned int address);

  //! Step one or more instructions
  /*!
      \param steps Number of instructions to step
  */
  void step(unsigned int steps);
  void step_over(void);

  //! Determines whether program has been modified 
  /*!
      (this is only valid for those processor capable of writing to their own
       program memory)
      \param address Address
      \return True if modified, false otherwise
  */
  bool isModified(unsigned int address);

  //! Given a file and a line in that file, find the instrucion in the processor's memory that's closest to it.
  /*!
      \param file_id File ID
      \param src_line Source line
      \return Address
  */
  int find_closest_address_to_line(int file_id, int src_line);
  //virtual int  find_closest_address_to_hll_line(int file_id, int src_line);

  // Given an address to an instruction, find the source line that 
  // created it:

  //! Gets the source line of address in program memory
  /*!
      \param address Program memory address
      \return Source line
  */
  unsigned int get_src_line(unsigned int address);

  //! Return the file ID of the source program responsible for the opcode at address.
  /*!
      \param address Program memory address
      \return File ID
  */
  unsigned int get_file_id(unsigned int address);

  // A couple of functions for manipulating  breakpoints
  //! Sets a break at address
  /*!
      \param address Address
  */
  void set_break_at_address(int address);
  //! Sets a read break at address
  /*!
      \param address Address
  */
  void set_read_break_at_address(int address);
  //! Sets a write break at address
  /*!
      \param address Address
  */
  void set_write_break_at_address(int address);
  //! Sets a notify at address
  /*!
      \param address Address
      \param cb BreakCallBack
  */
  void set_notify_at_address(int address, BreakCallBack *cb);
  void set_profile_start_at_address(int address, BreakCallBack *cb);
  void set_profile_stop_at_address(int address, BreakCallBack *cb);
  //! Clears break at address 
  /*!
      \param address 
      \param type Instruction type
      \return 1 if successful, 0 otherwise
  */
  int clear_break_at_address(int address,enum Instruction::INSTRUCTION_TYPES type);
  //! Clear notify at address
  /*!
      /param address Address
      \return 1 if successful, 0 otherwise
  */
  int clear_notify_at_address(int address);
  int clear_profile_start_at_address(int address);
  int clear_profile_stop_at_address(int address);
  //! Indicates whether an address has a break
  /*!
      \param address Program memory address
      \param type Instruction type
      \return 1 if has break, 0 otherwise
  */
  int address_has_break(int address,enum Instruction::INSTRUCTION_TYPES type=Instruction::BREAKPOINT_INSTRUCTION);
  //! Indicates whether an address has a notify
  /*!
      \param address Address
      \return 1 if has notify, 0 otherwise
  */
  int address_has_notify(int address);
  int address_has_profile_start(int address);
  int address_has_profile_stop(int address);
  //! Finds an instruction at given address
  /*!
      \param address Program memory address
      \param type Instruction type
  */
  Instruction *find_instruction(int address, enum Instruction::INSTRUCTION_TYPES type);
  //! Toggle break at address
  /*!
      \param address Address
  */
  void toggle_break_at_address(int address);
  //! Toggle read break at address
  /*!
      \param address Register address
  */
  void toggle_read_break_at_address(int address);
  //! Toggle write break at address
  /*!
      \param address Register address
  */
  void toggle_write_break_at_address(int address);
  //! Sets breakpoint at source line
  /*!
      \param file_id File ID
      \param src_line Source line number
  */
  void set_break_at_line(int file_id, int src_line);
  //! Clears a breakpoint at source line
  /*!
      \param file_id File ID
      \param src_line Source line number
  */
  void clear_break_at_line(int file_id, int src_line);
  //! Toggles a breakpoint at source line
  /*!
      \param file_id File ID
      \param src_line Source line number
  */
  void toggle_break_at_line(int file_id, int src_line);

  //void set_break_at_hll_line(int file_id, int src_line);
  //void clear_break_at_hll_line(int file_id, int src_line);
  //void toggle_break_at_hll_line(int file_id, int src_line);

  void set_hll_mode(int);
  enum HLL_MODES get_hll_mode(void) { return hll_mode;}
  bool isHLLmode(void) {return get_hll_mode() == HLL_MODE;}
};


//---------------------------------------------------------

//! \class
/*!
   \brief The RegisterMemoryAccess class is the interface used
   by objects other than the simulator to manipulate the 
   cpu's register memory.
*/
class RegisterMemoryAccess
{
 public:
  //! Default constructor
  RegisterMemoryAccess(void);
  //! Gets a register at address
  /*!
      \param address Register address
      \return Register
  */
  virtual Register *get_register(unsigned int address);
  //! Gets the size of register memory
  /*!
      \return Number of registers
  */  
  int get_size(void) { return nRegisters; }
  //! Sets the cpu
  /*!
      \param p Processor object
  */
  void set_cpu(Processor *p);
  //! Set registers 
  /*!
      \param _registers Register array
      \param _nRegisters Number of registers
  */
  void set_Registers(Register **_registers, int _nRegisters);
  //! Indicates whether a register has a breakpoint
  /*!
      \param address Register address
  */
  bool hasBreak(int address);

  //private:
  //! Number of registers 
  int nRegisters;
  //! Indicates if register memory is initialized
  bool initialized;
  //! Pointer to the array of registers.
  Register **registers;   
  //! Pointer to the processor whose registers are being managed. 
  Processor *cpu;             

};

//! \class
/*!
    \brief  Configuration modes.
    The configuration mode bits are the config word bits remapped.
    The remapping removes processor dependent bit definitions.
*/
class ConfigMode {
 public:

  //! \enum
  /*!
      \brief Config Register bits
  */
  enum {
    //! FOSC0 and  FOSC1 together define the PIC clock  
    CM_FOSC0 = 1<<0,  
    // All PICs todate have these two bits, but the ones with internal oscillators use them differently
    CM_FOSC1 = 1<<1,   
    // Watch dog timer enable                    
    CM_WDTE =  1<<2,  
    // Code Protection  
    CM_CP0 =   1<<3,  
    // Code Protection
    CM_CP1 =   1<<4,
    // Power on/Reset timer enable  
    CM_PWRTE = 1<<5,
    // Brown out detection enable
    CM_BODEN = 1<<6,  
    // CPD
    CM_CPD =   1<<7,
     // MCLR enable    
    CM_MCLRE = 1<<8,    
    // Hack for internal oscillators 
    CM_FOSC1x = 1<<9,   
  };

  //! Config mode
  int config_mode;
  //! Valid bits
  int valid_bits;
  
  //! Constructor for ConfigMode
  ConfigMode(void) { 
    config_mode = 0xffff; 
    valid_bits = CM_FOSC0 | CM_FOSC1 | CM_WDTE;
  };

  //! Sets the config mode
  /*!
      \param new_value New value
  */
  void set_config_mode(int new_value) { config_mode = new_value & valid_bits;};
  //! Sets the valid bits
  /*!
      \param new_value New value
  */
  void set_valid_bits(int new_value) { valid_bits = new_value;};
  //! Sets the FOSC0
  void set_fosc0(void){config_mode |= CM_FOSC0;};
  //! Clears the FOSC0
  void clear_fosc0(void){config_mode &= ~CM_FOSC0;};
  //! Gets the FOSC0 value
  /*!
      \return 1 or 0
  */
  bool get_fosc0(void){return (config_mode & CM_FOSC0);};
  //! Sets the FOSC1
  void set_fosc1(void){config_mode |= CM_FOSC1;};
  //! Clears the FOSC1
  void clear_fosc1(void){config_mode &= ~CM_FOSC1;};
  //! Gets the FOSC1 value
  /*!
      \return 0 or 1
  */
  bool get_fosc1(void){return (0 != (config_mode & CM_FOSC1));};
  bool get_fosc1x(void){return (0 != (config_mode & CM_FOSC1x));};

  //! Sets CP0
  void set_cp0(void)  {config_mode |= CM_CP0;  valid_bits |= CM_CP0;};
  //! Clears CP0
  void clear_cp0(void){config_mode &= ~CM_CP0; valid_bits |= CM_CP0;};
  //! Gets CP0
  /*!
      \return 0 or 1
  */
  bool get_cp0(void)  {return (0 != (config_mode & CM_CP0));};
  //! Sets CP1
  void set_cp1(void)  {config_mode |= CM_CP1;  valid_bits |= CM_CP1;};
  //! Clears CP1
  void clear_cp1(void){config_mode &= ~CM_CP1; valid_bits |= CM_CP1;};
  //! Gets CP1
  /*!
      \return 0 or 1
  */
  bool get_cp1(void)  {return (0 != (config_mode & CM_CP1));};

  //! Enables the WDT
  void enable_wdt(void)  {config_mode |= CM_WDTE;};
  //! Disables the WDT
  void disable_wdt(void) {config_mode &= ~CM_WDTE;};
  //! Gets the WDTE value
  /*!
      \return 0 or 1
  */
  bool get_wdt(void)     {return (0 != (config_mode & CM_WDTE));};

  //! Enable the MCLRE
  void enable_mclre(void)  {config_mode |= CM_MCLRE;};
  //! Disables the MCLRE
  void disable_mclre(void) {config_mode &= ~CM_MCLRE;};
  //! Gets the value of MCLRE
  /*!
      \return 0 or 1
  */
  bool get_mclre(void)     {return (0 != (config_mode & CM_MCLRE));};

  //! Enables PWRTE
  void enable_pwrte(void)   {config_mode |= CM_PWRTE;  valid_bits |= CM_PWRTE;};
  //! Disables PWRTE
  void disable_pwrte(void)  {config_mode &= ~CM_PWRTE; valid_bits |= CM_PWRTE;};
  //! Gets PWRTE value
  /*!
      \return 0 or 1
  */
  bool get_pwrte(void)      {return (0 != (config_mode & CM_PWRTE));};
  //! Indicates whether PWRTE is valid
  /*!
      \return 1 if true, 0 otherwise
  */
  bool is_valid_pwrte(void) {return (0 != (valid_bits & CM_PWRTE));};

  //! Prints the contents of register memory
  void print(void);
};


//! \class
/*!
    \brief This class models the PIC16F84A processor
*/ 
class Processor
{
   public:
#define FILE_REGISTERS  0x100
#define EEPROM_SIZE     0x40
#define INTERRUPT_VECTOR 4

      //! Constructor
      /*!
          Creates a new Processor object and gives it a reference to the GUI
          \param ms MicroSim GUI
      */
      Processor(MicroSim * ms);
      //QSignal abortSig;
      //! MicroSim GUI object
      MicroSim *microSim;
      
      //! Array of registers
      Register** registers;
      //! Array of registers (bank)
      Register** register_bank;   
      //! Program memory
      Instruction** program_memory;
      //! A bad instruction
      Invalid_Instruction* bad_instruction;
      //unsigned int program_memory_size;
      //! The simulation mode
      SIMULATION_MODES simulation_mode;
      //! Program Counter
      Program_Counter *pc;
      //! Hardware Mapper 
      HardwareMapper *hm;
      
      //! A dynamically allocated array for src file info
      struct file_context *files; 
      //! The number of elements allocated to that array 
      int number_of_source_files;  
      //! ID of lst file
      int lst_file_id;

      //! An identifier to differentiate this instantiation from others
      int processor_id;              

      // Oscillator frequency
      double frequency;
      // Oscillator period
      double period; 
      //! Vdd   
      double Vdd;
      //! WDT timeout
      double nominal_wdt_timeout;
      //! Indicates if using IO
      bool usingIO;

      //! WDT
      WDT          wdt;

      //! INDF
      INDF         *indf;
      //! FSR
      FSR          *fsr;
      //! Stack
      Stack         *stack;
      //! Status Register
      STATUS_Register *status;
      //! W Register
      WREG          *W;
      //! Option Register
      OPTION_REG   option_reg;
      //! PCL Register
      PCL          *pcl;
      //! PCLATH Register
      PCLATH       *pclath;
      
      //why we need two, i don't know
      //! INTCON register
      INTCON       intcon_reg;
      //! Pointer to INTCON
      INTCON       *intcon;
      //! PORT A
      PIC_IOPORT   *porta;
      //! TRISA
      IOPORT_TRIS  trisa;
      //! PORT B
      PIC_IOPORT   *portb;
      //! TRISB
      IOPORT_TRIS  trisb;
      
      //! Program memory access
      ProgramMemoryAccess  pma;
      //! Register memory access
      RegisterMemoryAccess rma;
      //! EEPROM memory access
      RegisterMemoryAccess ema;
      
      //! Timer 0
      TMR0         tmr0;
      //! EEPROM. Set to NULL for PIC's that don't have a data EEPROM
      EEPROM      *eeprom;       
      
      //! Configuration Word as read from hex or cod file
      unsigned int config_word;    
      //! Processor dependent configuration bits. 
      ConfigMode   *config_modes;   
      
      // Creation and manipulation of program memory
      //! Initializes program memory
      /*!
          \param memory_size Memory size
      */
      void init_program_memory(unsigned int memory_size);
      //! Initalizes program memory
      /*!
          \param address Program memory address
          \param value Value
      */
      void init_program_memory(int address,int value);
      //! Builds program memory in a specified range
      /*!
          \param memory Program memory array
          \param minaddr Min. address
          \param maxaddr Max. address
      */
      void build_program_memory(int *memory,int minaddr,int maxaddr);
      //! Gets the program memory size
      /*!
          \return Program memory size
      */
      unsigned int program_memory_size() {return 0x400;};
      //! Loads a hex file
      /*!
          \param hex_file Filename
      */
      void load_hex(const char* hex_file);
      void listProgramMemory();
      void listRegisters();
      
      // Creation and manipulation of registers
      //! Creates the invalid registers
      void create_invalid_registers (void);
      //! Returns the register memory size
      /*!
          \return Register memory size
      */
      unsigned int register_memory_size () const { return FILE_REGISTERS;};
      //! Adds the file registers in a specified range
      /*!
          \param start_address Start
          \param end_address End
          \param alias_offset Offset
      */
      void add_file_registers(unsigned int start_address, 
			  unsigned int end_address, 
			  unsigned int alias_offset);
      //! Deletes file registers in a specified range
      /*!
          \param start_address Start
          \param end_address End
      */
      void delete_file_registers(unsigned int start_address, 
			     unsigned int end_address);
	  //! Aliases file registers in a specified range
      /*!
          \param start_address Start
          \param end_address End
          \param alias_offset Offset
      */
      void alias_file_registers(unsigned int start_address, 
			    unsigned int end_address, 
			    unsigned int alias_offset);
      //! Initializes program memory
      /*!
          \param memory_size Register memory size
      */
      void init_register_memory(unsigned int memory_size);

      int  map_pm_address2index(int address) {return address;};
      int  map_pm_index2address(int index) {return index;};

      // Symbolic debugging

      //! Stores a source file line
      /*!
          \param address Program memory address
          \param file_id File ID
          \param sline Source line
          \param lst_line LST line
      */
      void attach_src_line(int address,int file_id,int sline,int lst_line);
      //! Reads the source files
      void read_src_files(void);
      //! Clears the source files currently loaded
      void clear_src_files(void);

      //! Return the portion of pclath that is used during branching instructions
      /*!
          \return Portion of PCLATH used in branches
      */
      unsigned int get_pclath_branching_jump(void)
      {
         return ((pclath->value & 0x18)<<8);
      }

      //! Return the portion of pclath that is used during modify PCL instructions
      /*!
          \return Portion of PCLATH used during modify PCL instructions
      */
      unsigned int get_pclath_branching_modpcl(void)
      {
         return((pclath->value & 0x1f)<<8);
      }
      //! Returns the FSR value
      /*!
          \return FSR value
      */
      unsigned int map_fsr_indf ( void )
      {
         return ( this->fsr->value );
      }

      //! Adds a special function register to address
      /*!
          \param reg SFR Register
          \param addr Register address
          \param por_value Value at POR
          \param new_name New name
      */
      void add_sfr_register(SFR_Register *reg, unsigned int addr,
			unsigned int por_value=0,char *new_name=NULL);
      //  virtual void create_symbols(void);
      //! Creates the Stack
      void create_stack(void) {stack = new Stack;};
  
  
      //! Runs the processor
      void run(void);
      //! Runs the processor with specified number of instructions
      /*!
          \param n Number of instructions to execute
      */
      void run(int n);
      //! Sleep
      void sleep(void);
      //! Step through a number of instructions
      /*!
          \param steps Number of instructions to step
      */
      void step(unsigned int steps);
      //! Steps over instructions
      void step_over(void);
      //! Stops the processor
      void stop();

      //! Steps once
      void step_one(void);

      //! Causes an interrupt
      void interrupt(void); 
      void pm_write(void);

      /*Config Mode functions - DO LATER*/
      //! Sets the Config Word
      /*!
          \param address Address
          \param cfg_word Value
      */
      void set_config_word(unsigned int address, unsigned int cfg_word);
      //! Gets the Config Word
      /*!
          \return Config word value
      */
      unsigned int get_config_word(void) {return config_word;};
      //! Gets the config word address
      /*!
          \return 0x2007
      */
      unsigned int config_word_address(void) const {return 0x2007;};
      //! Creates a Config Word
      /*!
          \return Config Word object
      */
      ConfigMode *create_ConfigMode(void) { return new ConfigMode; };

      //! Sets the frequency
      /*!
          \param f frequency
      */
      void set_frequency(double f);
      //! Time to cycles
      /*!
          \param t time
          \return Time to cycles
      */
      unsigned int time_to_cycles(double t);
      
      //! Resets processor
      /*!
          \param r Reset type
      */
      void reset(RESET_TYPE r);
      //! Disassembles in a specified range
      /*!
          \param start_address Start
          \param end_address End
      */
      void disassemble (int start_address, int end_address);
      //void list(int file_id, int pcval, int start_line, int end_line);

      //! Power on reset
      void por(void);
      //! Creates the processor
      void create(void);
      //! Creates the IO pin map
      void create_iopin_map(void);
      
      //! Disassembles an instruction
      /*!
          \param address Address
          \param inst Mask
          \return Instruction
      */
      Instruction * disasm_cpu ( unsigned int address,unsigned int inst)
      {
         return disasm(this,inst);
      };


      //  void option_new_bits_6_7(unsigned int)=0;
      //! Gets the FSR value
      /*!
          \param load_value Load value
          \return Load value
      */  
      unsigned int get_fsr_value ( unsigned int load_value )
      {
         return load_value;
      };
  
      /*EEPROM functions - do later*/
      //! Sets the EEPROM for this processor
      /*!
          \param e EEPROM object
      */
      void set_eeprom(EEPROM *e);
      //! Gets the EEPROM 
      /*!
          \return EEPROM object
      */
      EEPROM *get_eeprom(void) { return (eeprom); }


      
};

//! \class
/*!
    \brief This class models the Program Counter
*/
class Program_Counter
{
public:
  //! Processor this PC belongs to
  Processor *cpu;

  //! PC's current value 
  unsigned int value;        
  //! Memory size mask    
  unsigned int memory_size_mask; 
  //! PCLATH confines PC to banks 
  unsigned int pclath_mask;     
  //! Instruction phase  
  unsigned int instruction_phase;

  //! Default constructor
  Program_Counter(void);
  //! Increments PC
  virtual void increment(void);
  //! Skips an instruction
  virtual void skip(void);
  //! Jumps to a new address
  /*!
      \param new_value New address
  */
  virtual void jump(unsigned int new_value);
  //! Update the PC except PCLATH is ignored
  /*!
      \param new_value New address
  */
  virtual void interrupt(unsigned int new_value);
  //! Update the program counter. 
  /*!
      Anytime the pcl register is written to by the source code we'll pass through here.
      \param new_value New address
  */ 
  virtual void computed_goto(unsigned int new_value);
  //! Write a new value to the program counter. All returns pass through here.
  /*!
      \param new_value New address
  */ 
  virtual void new_address(unsigned int new_value);
  //! Change the program counter without affecting the cycle counter
  /*!
      \param new_value New address
  */ 
  virtual void put_value(unsigned int new_value);
  //! Gets value of PC
  /*!
      \return PC value
  */
  virtual unsigned int get_value(void) {
    return value;
  };
  //! Gets the raw value of PC
  /*!
     get_value is multiplied by 2 whereas get_raw_value isn't. 
     The raw value of the program counter is used as an index into the program memory.
     \return value
  */
  virtual unsigned int get_raw_value(void) {
    return value;
  }

  //! Sets the phase
  /*!
      \param phase Phase
  */
  virtual void set_phase(int phase) { instruction_phase = phase;}
  //! Gets the phase
  /*!
      \return Phase
  */
  virtual int get_phase(void) {return instruction_phase; }

  //! Sets the reset address
  /*!
      \param _reset_address Reset Address
  */
  void set_reset_address(unsigned int _reset_address)
    {
      reset_address = _reset_address;
    }
  //! Gets the reset address
  /*!
      \return Reset address
  */
  unsigned int get_reset_address(void) 
    {
      return reset_address;
    }

  //! Resets the PC
  void reset(void);

  //! Get the next address that is just pass the current one
  /*!
      \return Next address
  */
  virtual unsigned int get_next(void);

  //XrefObject *xref;

private:
  //! Value pc gets at reset
  unsigned int reset_address;      
  
};

/*
#define cpu_pic ( (pic_processor *)cpu)


//----------------------------------------------------------
// Global definitions:

extern Processor *active_cpu;

//--------------------------------------
//
// non-class helper functions.

void initialize_processor_constructor(void);
Processor *  add_processor(char * processor_type, char * processor_new_name);
void display_available_processors(void);
void dump_processor_list(void);
int find_in_available_processor_list(char * processor_type);
Processor *get_processor(unsigned int cpu_id);*/


#endif
