#ifndef REGISTER_H
#define REGISTER_H

#include "enum.h"
//#include "breakpoints.h"
#include "cycles.h"
#include <cstdlib>
#include <string>
#include <iostream>

using namespace std;
class Processor;
class symbol;


//! \class
/*!
    \brief This is the base class for a register
*/
class Register
{
   public:
      #define ALIAS_MASK 0x80
      #define BIT_MASK = 7
      //! \enum
      /*!
          \brief Register types
      */
      enum REGISTER_TYPES
      {
         //! Invalid Register
         INVALID_REGISTER,
         //! Generic Register
         GENERIC_REGISTER,
         //! File Register
         FILE_REGISTER,
         //! SFR Register
         SFR_REGISTER,
         //! BP Register
         BP_REGISTER
      };

      //! Name
      char *name_str1;
      //! Value
      unsigned int value;
      //! Address
      unsigned int address;
      //! Alias mask
      unsigned int alias_mask;
      //! Power on reset value
      unsigned int por_value;  
      //! Symbol alias
      symbol *symbol_alias;
      //! Processor
      Processor * cpu;
      //! Default constructor
      Register();
      //! Destructor
      virtual ~Register();
      //! Return the type of register 
      /*!
          \return Register type (GENERIC_REGISTER)
      */
      virtual REGISTER_TYPES isa(void) {return GENERIC_REGISTER;};
      //! Returns the value 
      /*!
          \return Value
      */
      virtual unsigned int get_value(void) {return value;};
      //! Puts a new value
      /*!
          \param new_value New value
      */
      virtual void put_value(unsigned int new_value) {value = new_value;};
      //! Gets the value
      /*!
          \return value
      */
      virtual unsigned int get(void) {return value;};
      //! Puts the value
      /*!
          \param new_value New value
      */
      virtual void put(unsigned int new_value) {value = new_value;};
      //! Gets the name
      /*!
          \return name
      */
      char* name(void) {return name_str1;};
      //! Creates a new name for register
      void new_name(char* new_name);
      //! Resets the register
      /*!
          \param r Reset type
      */
      virtual void reset(RESET_TYPE r) { if(r == POR_RESET) value = 0x00; };
      //! Sets a bit 
      /*!
          \param bit_number Bit
          \param new_value Bit value
      */
      virtual void setBit(unsigned int bit_number, bool new_value);
      //! Gets the bit value
      /*!
          \param bit_number Bit
          \return Bit value
      */
      virtual int getBit(unsigned int bit_number);
};


//------------------------------------------------------------
//! \class
/*!
    \brief This class models a File Register
*/
class File_Register : public Register
{
 public:

   //! Default constructor
   File_Register(void):Register(){};
   //~File_Register(void){};
   //! Return the type of register 
   /*!
       \return Register type (FILE_REGISTER)
   */
   virtual REGISTER_TYPES isa(void) {return FILE_REGISTER;};  
  

};

//! \class
/*!
    \brief This class models an Invalid File Register
*/
class Invalid_File_Register : public File_Register
{
public:

   //virtual void put(unsigned int new_value);
   //virtual unsigned int get(void);
   //! Return the type of register 
   /*!
       \return Register type (INVALID_REGISTER)
   */
   virtual REGISTER_TYPES isa(void) {return INVALID_REGISTER;}; 
   //! Constructor 
   /*!
       \param at_address Address
   */
   Invalid_File_Register(unsigned int at_address);
};


//---------------------------------------------------------
//! \class
/*!
   \brief  Base class for a special function register
*/
class SFR_Register : public File_Register
{
 public:
  //! Default constructor
  SFR_Register() :File_Register(){}
  //! Wdt or mclr reset value
  unsigned int wdtr_value; 

  //! Initialize SFR 
  virtual void initialize(void) {return;};
  //! Return the type of register 
   /*!
       \return Register type (SFR_REGISTER)
   */
  virtual REGISTER_TYPES isa(void) {return SFR_REGISTER;};
  //! Reset the register
  /*!
      \param r Reset type
  */
  virtual void reset(RESET_TYPE r) {
    switch (r) {

    case POR_RESET:
      value = por_value;
      break;

    case WDT_RESET:
      value = wdtr_value;
      break;
    case SOFT_RESET:
      break;
    }

  }
};


//---------------------------------------------------------
// FSR register
//

//! \class
/*!
    \brief This class models the FSR Register
*/
class FSR : public SFR_Register
{
public:
   //! Creates FSR
   FSR(void);
};

//---------------------------------------------------------
// Status register
//

//! \class
/*!
    \brief This class models the Status Register
*/
class STATUS_Register : public SFR_Register
{
public:

#define STATUS_Z_BIT   2
#define STATUS_C_BIT   0
#define STATUS_DC_BIT  1
#define STATUS_PD_BIT  3
#define STATUS_TO_BIT  4
#define STATUS_Z       (1<<STATUS_Z_BIT)
#define STATUS_C       (1<<STATUS_C_BIT)
#define STATUS_DC      (1<<STATUS_DC_BIT)
#define STATUS_PD      (1<<STATUS_PD_BIT)
#define STATUS_TO      (1<<STATUS_TO_BIT)
//#define BREAK_Z_ACCESS 2
//#define BREAK_Z_WRITE  1

#define RP_MASK        0x20
  //unsigned int break_point;
  //unsigned int break_on_z,break_on_c;
  //! RP mask
  unsigned int rp_mask;
  //! Bits that instructions can modify
  unsigned int write_mask;   

  //! Default constructor
  STATUS_Register(void);


  //! Puts a new value
  /*!
      \param new_value New value
  */
  inline void put(unsigned int new_value);
  //! Puts a new value
  /*!
      \param new_value New value
  */
  inline void put_value(unsigned int new_value);

  //! Gets the value
  /*!
      \return value
  */
  inline unsigned int get(void)
    {
      /*if(break_point) 
	bp.check_read_break(this);

      trace.register_read(address,value);*/
      return(value);
    }

  //! Special member function to control just the Z bit
  /*!
      \param new_z New Z bit value
  */
  inline void put_Z(unsigned int new_z)
    {
      /*if(break_on_z) 
	bp.check_write_break(this);*/

      value = (value & ~STATUS_Z) | ((new_z) ? STATUS_Z : 0);

      //trace.register_write(address,value);
    }

  //! Gets the Z bit 
  /*!
      \return Z bit value
  */
  inline unsigned int get_Z(void)
    {
      //if(break_on_z) 
	//bp.check_read_break(this);

      //trace.register_read(address,value);
      return( ( (value & STATUS_Z) == 0) ? 0 : 1);
    }


  //! Special member function to control just the C bit
  /*!
      \param new_c New C bit value
  */
  void put_C(unsigned int new_c)
    {
      //if(break_on_c) 
	//bp.check_write_break(this);

      value = (value & ~STATUS_C) | ((new_c) ? STATUS_C : 0);
      //trace.register_write(address,value);
    }

  //! Gets the C bit 
  /*!
      \return C bit value
  */
  unsigned int get_C(void)
    {
      //if(break_on_c) 
	//bp.check_read_break(this);

      //trace.register_read(address,value);
      return( ( (value & STATUS_C) == 0) ? 0 : 1);
    }

  //! Special member function to set Z, C, and DC
  /*!
      \param new_value New value
      \param src1 Source 1
      \param src2 Source 2
  */
  inline void put_Z_C_DC(unsigned int new_value, unsigned int src1, unsigned int src2)
    {
      //if(break_point) 
	//bp.check_write_break(this);

      value = (value & ~ (STATUS_Z | STATUS_C | STATUS_DC)) |  
	((new_value & 0xff)   ? 0 : STATUS_Z)   |
	((new_value & 0x100)  ? STATUS_C : 0)   |
	(((new_value ^ src1 ^ src2)&0x10) ? STATUS_DC : 0);

      //trace.register_write(address,value);
    }

  //! Special member function to set Z, C, and DC for SUB
  inline void put_Z_C_DC_for_sub(unsigned int new_value, unsigned int src1, unsigned int src2)
    {
      //if(break_point) 
	//bp.check_write_break(this);

      value = (value & ~ (STATUS_Z | STATUS_C | STATUS_DC)) |  
	((new_value & 0xff)   ? 0 : STATUS_Z)   |
	((new_value & 0x100)  ? 0 : STATUS_C)   |
	(((new_value ^ src1 ^ src2)&0x10) ? 0 : STATUS_DC);

      //trace.register_write(address,value);
    }

  //! Puts a new PD value
  /*!
      \param new_pd New PD value
  */
  inline void put_PD(unsigned int new_pd)
    {
      //if(break_point) 
	//bp.check_write_break(this);

      value = (value & ~STATUS_PD) | ((new_pd) ? STATUS_PD : 0);

      //trace.register_write(address,value);
    }

  //! Gets the PD value
  /*!
      \return PD value
  */
  inline unsigned int get_PD(void)
    {
      //if(break_point) 
	//bp.check_read_break(this);

      //trace.register_read(address,value);
      return( ( (value & STATUS_PD) == 0) ? 0 : 1);
    }

  //! Puts a new TO value
  /*!
      \param new_to New TO value
  */
  inline void put_TO(unsigned int new_to)
    {
      //if(break_point) 
	//bp.check_write_break(this);

      value = (value & ~STATUS_TO) | ((new_to) ? STATUS_TO : 0);

      //trace.register_write(address,value);
    }
  //! Gets the TO value
  /*!
      \return TO value
  */
  inline unsigned int get_TO(void)
    {
      //if(break_point) 
	//bp.check_read_break(this);

      //trace.register_read(address,value);
      return( ( (value & STATUS_TO) == 0) ? 0 : 1);
    }

  
  

};


//---------------------------------------------------------
// W register

//! \class
/*!
    \brief This class models the W Register
*/
class WREG : public SFR_Register
{
public:

  //! Puts a new value
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  //! Reset the register
  /*!
      \param r Reset type
  */
  void reset(RESET_TYPE r) {if(r == POR_RESET) value = 0x00;};
  //! Gets the value
  /*!
      \return value
  */
  unsigned int get(void);
  //! Default constructor
  WREG(void);
};

//---------------------------------------------------------
// INDF

//! \class
/*!
    \brief This class models the INDF register
*/
class INDF : public SFR_Register
{
public:
  #define FSR_ADDR 4 //for P16f84a
  //! FSR mask
  unsigned int fsr_mask;
  //! Base address mask 1
  unsigned int base_address_mask1;
  //! Base address mask 2
  unsigned int base_address_mask2;

  //! Default constructor
  INDF(void);
  //! Puts a new value
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  //virtual void put_value(unsigned int new_value);
  //! Gets the value
  /*!
      \return value
  */
  unsigned int get(void);
  //unsigned int get_value(void);
};


//---------------------------------------------------------
// OPTION_REG - 

//! \class
/*!
    \brief This class models the OPTION register
*/
class OPTION_REG : public SFR_Register
{
public:

enum
  {
    PS0    = 1<<0,
    PS1    = 1<<1,
    PS2    = 1<<2,
    PSA    = 1<<3,
    T0SE   = 1<<4,
    T0CS   = 1<<5,
    BIT6   = 1<<6,
    BIT7   = 1<<7
  };

  //! Prescale
  unsigned int prescale;

  //! Default constructor
  OPTION_REG(void);

  //! Gets the prescale
  /*!
      \return Prescale
  */
  inline unsigned int get_prescale(void)
    {
      return value & (PS0 | PS1 | PS2);
    }

  //! Gets the PSA
  /*!
      \return PSA
  */
  inline unsigned int get_psa(void)
    {
      return value & PSA;
    }

  //! Gets the T0CS
  /*!
      \return T0CS
  */
  inline unsigned int get_t0cs(void)
    {
      return value & T0CS;
    }

  //! Gets the T0SE
  /*!
      \return T0SE
  */
  inline unsigned int get_t0se(void)
    {
      return value & T0SE;
    }
  //! Puts a new value
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  //! Puts a new value
  /*!
      \param new_value New value
  */
  void put_value(unsigned int new_value);

};

//---------------------------------------------------------
// PCL - Program Counter Low
//

//! \class
/*!
    \brief This class models the PCL register
*/
class PCL : public SFR_Register
{
public:

  //! Puts a new value
  /*!
      \param new_value New value
  */
  virtual void put(unsigned int new_value);
  //! Puts a new value
  /*!
      \param new_value New value
  */
  virtual void put_value(unsigned int new_value);
  //! Gets the value
  /*!
      \return value
  */
  virtual unsigned int get(void);
  //! Gets the value
  /*!
      \return value
  */
  virtual unsigned int get_value(void);

  //! Default constructor
  PCL(void);
};




//---------------------------------------------------------
// PCLATH - Program Counter Latch High
//

//! \class
/*!
    \brief This class models the PCLATH register
*/
class PCLATH : public SFR_Register
{
public:
  #define PCLATH_MASK 0x1f
  //! Puts a new value
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);
  //! Puts a new value
  /*!
      \param new_value New value
  */
  void put_value(unsigned int new_value);
  //! Gets the value
  /*!
      \return value
  */
  unsigned int get(void);

  //! Default constructor
  PCLATH(void);
};



//---------------------------------------------------------
// PCON - Power Control/Status Register
//
//! \class
/*!
    \brief This class models the PCON register
*/
class PCON : public SFR_Register
{
 public:

  enum {
    BOR = 1<<0,   // Brown Out Reset
    POR = 1<<1    // Power On Reset
  };

  //! Valid bits
  unsigned int valid_bits;

  //! Puts a new value
  /*!
      \param new_value New value
  */
  void put(unsigned int new_value);

  //! Default constructor
  PCON(void);
};

//---------------------------------------------------------
// INTCON - Interrupt control register

//! \class
/*!
    \brief This class models the INTCON register
*/
class INTCON : public SFR_Register
{
public:

enum
{
  RBIF = 1<<0,
  INTF = 1<<1,
  T0IF = 1<<2,
  RBIE = 1<<3,
  INTE = 1<<4,
  T0IE = 1<<5,
  XXIE = 1<<6,    // Processor dependent
  GIE  = 1<<7
};


  //! Default constructor
  INTCON(void);
  //! Sets the T0IF
  void set_T0IF(void);

  /*
  // Bit 6 of intcon depends on the processor that's being simulated, 
  // This generic function will get called whenever interrupt flag upon
  // which bit 6 enables becomes true. (e.g. for the c84, this
  // routine is called when EEIF goes high.)
  */
  //! Causes peripheral interrupt
  void peripheral_interrupt(void);

  //! Sets GIE
  inline void set_gie(void)
    {
      value |= GIE;
      put(value);
    }

  //! Sets RBIF
  inline void set_rbif(void)
    {
      put(get_value() | RBIF);
    }

  //! Sets INTF
  inline void set_intf(void)
    {
      put(get_value() | INTF);
    }

  //! Sets T0IF
  inline void set_t0if(void)
    {
      put(get_value() | T0IF);
    }

  //! Sets RBIE
  inline void set_rbie(void)
    {
      put(get_value() | RBIE);
    }

  //! Sets INTE
  inline void set_inte(void)
    {
      put(get_value() | INTE);
    }

  //! Sets T0IE
  inline void set_t0ie(void)
    {
      put(get_value() | T0IE);
    }

  //! Clears GIE
  inline void clear_gie(void)
    {
      put(get_value() & ~GIE);
    }

  //! Checks peripheral interrupts
  /*!
      \return 0
  */
  virtual bool check_peripheral_interrupt(void)
    {
      return 0;
    }
  //! Puts a new value
  /*!
      \param new_value New value
  */
  virtual void put(unsigned int new_value);

};


class Stack
{
public:
  //! Stack array 
  unsigned int contents[32];   
  //! Stack pointer 
  unsigned int pointer;  
  //! 1 for 12bit, 7 for 14bit, 31 for 16bit                 
  unsigned int stack_mask;    
  //! Warnings on stack over/under flow   
  bool stack_warnings_flag; 
  //! Break on overflow
  bool break_on_overflow;  
  //! Break on underflow     
  bool break_on_underflow;         

  //! Default constructor
  Stack(void);
  //! Pushes a value onto the stack
  /*!
      \param value Value
  */
  virtual void push(unsigned int value);
  //! Pops a value off the stack
  /*!
      \return Value
  */
  virtual unsigned int pop(void);
  //! Resets the Stack pointer
  virtual void reset(void) {pointer = 0;};  // %%% FIX ME %%% reset may need to change 
  // because I'm not sure how the stack is affected by a reset.
  //! Sets a break on overflow
  /*!
      \param clear_or_set True for set, false for unset
  */
  virtual bool set_break_on_overflow(bool clear_or_set);
  //! Sets a break on underflow
  /*!
      \param clear_or_set True for set, false for unset
  */
  virtual bool set_break_on_underflow(bool clear_or_set);

};




#endif
