/*
   Copyright (C) 1998 T. Scott Dattalo

This file is part of gpsim.

gpsim is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

gpsim is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with gpasm; see the file COPYING.  If not, write to
the Free Software Foundation, 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

//
// symbol.h
//

#include <iostream>
#include <string>
#include "enum.h"
#include "register.h"
#include "processor.h"

using namespace std;

#ifndef __SYMBOL_H__
#define __SYMBOL_H__

int load_symbol_file(Processor **, const char *);
void display_symbol_file_error(int);

//#include "gpsim_interface.h"

//! \define
/*!
    \brief Symbol Types
*/
enum SYMBOL_TYPE
{
  //! Invalid
  SYMBOL_INVALID,
  //! Base class
  SYMBOL_BASE_CLASS,
  //! IOPort
  SYMBOL_IOPORT,
  //! Stimulus Node
  SYMBOL_STIMULUS_NODE,
  //! Stimulus
  SYMBOL_STIMULUS,
  //! Line Number
  SYMBOL_LINE_NUMBER,
  //! Constant
  SYMBOL_CONSTANT,
  //! Register
  SYMBOL_REGISTER,
  //! Address
  SYMBOL_ADDRESS,
  //! Special Register
  SYMBOL_SPECIAL_REGISTER,
  //! Processor
  SYMBOL_PROCESSOR,
  //! Module
  SYMBOL_MODULE
};

//class stimulus;
//class Stimulus_Node;
//class WREG;
//class IOPORT;
//class Processor;
//class Register;
//class Module;

//! \class
/*!
    \brief Symbol Type
*/
class symbol_type
{
public:
  //! Symbol type
  SYMBOL_TYPE type;
  //! Name 
  char * name_str;
};

extern symbol_type symbol_types[];

//! \class
/*!
    \brief This class models a Symbol
*/
class symbol
{
public:

  //! Symbol name
  string name_str;
  //! Processor
  Processor *cpu;
  //! Returns type of symbol
  /*!
      \return Symbol type
  */
  virtual SYMBOL_TYPE isa(void) { return SYMBOL_BASE_CLASS;};
  //! Returns the type name
  /*!
      \return Type name string
  */
  char * type_name(void) { return symbol_types[isa()].name_str;};
  //! Returns name of symbol
  /*!
      \return Name string
  */
  string *name(void) { return &name_str;};
  //! Assigns a new name to symbol
  /*!
      \param new_name_str New name string
  */
  void new_name(string *new_name_str) {name_str = *new_name_str;};
  //! Print symbol
  virtual void print(void);
  //! Gets the value of symbol
  /*!
      \return 0
  */
  virtual int get_value(void){return 0;};
  //! Puts value into symbol
  virtual void put_value(int i) {;};
  //! Default constructor
  symbol(void);
  //! Destructor
  virtual ~symbol() {};

};

//! \class
/*!
    \brief This class is a collection of symbols
*/
class Symbol_Table
{
public:
  //void add_ioport(Processor *cpu, IOPORT *ioport);
  //void add_stimulus_node(Stimulus_Node *stimulus_node);
  //void add_stimulus(stimulus *s);
  //! Adds a line number to table
  /*!
      \param cpu Processor
      \param address Address
  */
  void add_line_number(Processor *cpu, int address);
  //! Adds a constant to table
  /*!
      \param cpu Processor
      \param c constant
  */
  void add_constant(Processor *cpu, char *, int c);
  //! Adds a register to table
  /*!
      \param cpu Processor
      \param reg Register
  */
  void add_register(Processor *cpu, Register *reg);
  //! Adds an address to table
  /*!
      \param cpu Processor
      \param addr Address
  */
  void add_address(Processor *cpu, char *, int addr);
  //! Adds W Reg to table
  /*!
      \param cpu Processor
      \param w W Register
  */
  void add_w(Processor *cpu, WREG *w );
  //void add_module(Module * m, const char *module_name);
  //void remove_module(Module * m, char *module_name);
  //! Adds a symbol to table
  /*!
      \param cpu Processor
      \param symbol_name Name
      \param symbol_type Type
      \param value Value
  */
  void add(Processor *cpu, char *symbol_name, char *symbol_type, int value);
  //! Dumps all symbols to screen
  void dump_all(void);
  //! Dumps symbol with name
  /*!
      \param s Name
  */
  void dump_one(char *s);
  //! Dumps symbol with name
  /*!
      \param s Name
  */
  void dump_one(string *s);
  //! Dumps symbol with type
  /*!
      \param symt Symbol type
  */
  void dump_type(SYMBOL_TYPE symt);
  
  //! Find symbol with name
  /*!
      \param s Name
  */
  symbol * find(char *s);
  //! Find symbol with name
  /*!
      \param s Name
  */
  symbol * find(string *s);
  //! Finds symbol with type
  /*!
      \param symt Symbol type
  */
  symbol * find(SYMBOL_TYPE symt, char *s);
};

extern Symbol_Table symbol_table;

//! \class
/*!
    \brief Constant Symbol
*/
class constant_symbol : public symbol
{
public:

  //! Destructor
  ~constant_symbol() {};
  //! Value
  int val;
  //! Returns symbol type
  /*!
      \return SYMBOL_CONSTANT
  */
  virtual SYMBOL_TYPE isa(void) { return SYMBOL_CONSTANT;};
  //! Prints symbol
  virtual void print(void);
  //! Gets the value of symbol
  /*!
      \return value
  */
  virtual int get_value(void){return val;};
};

//! \class
/*!
    \brief Register Symbol
*/
class register_symbol : public symbol
{
public:

  //! Destructor
  ~register_symbol() {};
  //! Register
  Register *reg;
  //! Returns symbol type
  /*!
      \return SYMBOL_REGISTER
  */
  virtual SYMBOL_TYPE isa(void) { return SYMBOL_REGISTER;};
  //! Prints symbol
  virtual void print(void);
  //! Gets the value of symbol
  /*!
      \return value
  */
  virtual int get_value(void);
  //! Puts value into symbol
  /*!
      \param new_value New value
  */
  virtual void put_value(int new_value);
};


//! \class
/*!
    \brief Address Symbol
*/
class address_symbol : public symbol
{
public:
  //! Destructor
  ~address_symbol() {};
  //! Value
  int val;
  //! Returns symbol type
  /*!
      \return SYMBOL_REGISTER
  */
  virtual SYMBOL_TYPE isa(void) { return SYMBOL_ADDRESS;};
  //! Prints symbol
  virtual void print(void);
  //! Gets the value of symbol
  /*!
      \return value
  */
  virtual int get_value(void){return val;};

};

//! \class
/*!
    \brief Line Number Symbol
*/
class line_number_symbol : public symbol
{
 public:

  //! Destructor
  ~line_number_symbol() {};
  //! Address
  int address;
  //! Source ID
  int src_id;
  //! Source line
  int src_line;
  //! LST ID
  int lst_id;
  //! LST Line
  int lst_line;
  //! LST Page
  int lst_page;
  //! Put address
  /*!
      \param new_address New address
  */
  void put_address(int new_address) {address = new_address;}
  //! Put source line
  /*!
      \param new_src_line New source line
  */
  void put_src_line(int new_src_line) {src_line = new_src_line;}
  //! Put LST line
  /*!
      \param new_lst_line New LST line
  */
  void put_lst_line(int new_lst_line) {lst_line = new_lst_line;}
  //! Put LST page
  /*!
      \param new_lst_page New LST page
  */
  void put_lst_page(int new_lst_page) {lst_page = new_lst_page;}
  //! Returns symbol type
  /*!
      \return SYMBOL_REGISTER
  */
  virtual SYMBOL_TYPE isa(void) { return SYMBOL_LINE_NUMBER;};
};



//! \class
/*!
   \brief Place W into the symbol table
*/
class w_symbol : public symbol
{
 public:

  //! Destructor
  ~w_symbol() {};
  //! W Reg
  WREG *w;

  //! Returns symbol type
  /*!
      \return SYMBOL_REGISTER
  */
  virtual SYMBOL_TYPE isa(void) { return SYMBOL_SPECIAL_REGISTER;};
  //! Prints symbol
  virtual void print(void);
  //! Gets the value of symbol
  /*!
      \return value
  */
  virtual int get_value(void);
  //! Puts value into symbol
  /*!
      \param new_value New value
  */
  virtual void put_value(int new_value);

};
#endif  //  __SYMBOL_H__
